# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pyweed/gui/uic/StationOptionsWidget.ui'
#
# Created by: PyQt4 UI code generator 4.11.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_StationOptionsWidget(object):
    def setupUi(self, StationOptionsWidget):
        StationOptionsWidget.setObjectName(_fromUtf8("StationOptionsWidget"))
        StationOptionsWidget.resize(302, 682)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(StationOptionsWidget.sizePolicy().hasHeightForWidth())
        StationOptionsWidget.setSizePolicy(sizePolicy)
        self.verticalLayout = QtGui.QVBoxLayout(StationOptionsWidget)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.SNCLGroupBox = QtGui.QGroupBox(StationOptionsWidget)
        self.SNCLGroupBox.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.SNCLGroupBox.setObjectName(_fromUtf8("SNCLGroupBox"))
        self.verticalLayout_5 = QtGui.QVBoxLayout(self.SNCLGroupBox)
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.formLayout = QtGui.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtGui.QFormLayout.FieldsStayAtSizeHint)
        self.formLayout.setFormAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.formLayout.setHorizontalSpacing(4)
        self.formLayout.setVerticalSpacing(1)
        self.formLayout.setObjectName(_fromUtf8("formLayout"))
        self.networkLabel = QtGui.QLabel(self.SNCLGroupBox)
        self.networkLabel.setObjectName(_fromUtf8("networkLabel"))
        self.formLayout.setWidget(0, QtGui.QFormLayout.LabelRole, self.networkLabel)
        self.networkLineEdit = QtGui.QLineEdit(self.SNCLGroupBox)
        self.networkLineEdit.setObjectName(_fromUtf8("networkLineEdit"))
        self.formLayout.setWidget(0, QtGui.QFormLayout.FieldRole, self.networkLineEdit)
        self.stationLineEdit = QtGui.QLineEdit(self.SNCLGroupBox)
        self.stationLineEdit.setObjectName(_fromUtf8("stationLineEdit"))
        self.formLayout.setWidget(1, QtGui.QFormLayout.FieldRole, self.stationLineEdit)
        self.locationLabel = QtGui.QLabel(self.SNCLGroupBox)
        self.locationLabel.setObjectName(_fromUtf8("locationLabel"))
        self.formLayout.setWidget(2, QtGui.QFormLayout.LabelRole, self.locationLabel)
        self.locationLineEdit = QtGui.QLineEdit(self.SNCLGroupBox)
        self.locationLineEdit.setObjectName(_fromUtf8("locationLineEdit"))
        self.formLayout.setWidget(2, QtGui.QFormLayout.FieldRole, self.locationLineEdit)
        self.channelLabel = QtGui.QLabel(self.SNCLGroupBox)
        self.channelLabel.setObjectName(_fromUtf8("channelLabel"))
        self.formLayout.setWidget(3, QtGui.QFormLayout.LabelRole, self.channelLabel)
        self.channelLineEdit = QtGui.QLineEdit(self.SNCLGroupBox)
        self.channelLineEdit.setObjectName(_fromUtf8("channelLineEdit"))
        self.formLayout.setWidget(3, QtGui.QFormLayout.FieldRole, self.channelLineEdit)
        self.stationLabel = QtGui.QLabel(self.SNCLGroupBox)
        self.stationLabel.setObjectName(_fromUtf8("stationLabel"))
        self.formLayout.setWidget(1, QtGui.QFormLayout.LabelRole, self.stationLabel)
        self.verticalLayout_5.addLayout(self.formLayout)
        self.verticalLayout.addWidget(self.SNCLGroupBox)
        self.timeGroupBox = QtGui.QGroupBox(StationOptionsWidget)
        self.timeGroupBox.setObjectName(_fromUtf8("timeGroupBox"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.timeGroupBox)
        self.verticalLayout_2.setContentsMargins(-1, -1, -1, 4)
        self.verticalLayout_2.setSpacing(4)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.verticalLayout_6 = QtGui.QVBoxLayout()
        self.verticalLayout_6.setSpacing(0)
        self.verticalLayout_6.setObjectName(_fromUtf8("verticalLayout_6"))
        self.formLayout_2 = QtGui.QFormLayout()
        self.formLayout_2.setFieldGrowthPolicy(QtGui.QFormLayout.FieldsStayAtSizeHint)
        self.formLayout_2.setFormAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.formLayout_2.setHorizontalSpacing(4)
        self.formLayout_2.setVerticalSpacing(1)
        self.formLayout_2.setObjectName(_fromUtf8("formLayout_2"))
        self.starttimeLabel = QtGui.QLabel(self.timeGroupBox)
        self.starttimeLabel.setObjectName(_fromUtf8("starttimeLabel"))
        self.formLayout_2.setWidget(0, QtGui.QFormLayout.LabelRole, self.starttimeLabel)
        self.starttimeDateTimeEdit = QtGui.QDateTimeEdit(self.timeGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.starttimeDateTimeEdit.sizePolicy().hasHeightForWidth())
        self.starttimeDateTimeEdit.setSizePolicy(sizePolicy)
        self.starttimeDateTimeEdit.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.starttimeDateTimeEdit.setCalendarPopup(True)
        self.starttimeDateTimeEdit.setTimeSpec(QtCore.Qt.UTC)
        self.starttimeDateTimeEdit.setObjectName(_fromUtf8("starttimeDateTimeEdit"))
        self.formLayout_2.setWidget(0, QtGui.QFormLayout.FieldRole, self.starttimeDateTimeEdit)
        self.endtimeLabel = QtGui.QLabel(self.timeGroupBox)
        self.endtimeLabel.setObjectName(_fromUtf8("endtimeLabel"))
        self.formLayout_2.setWidget(1, QtGui.QFormLayout.LabelRole, self.endtimeLabel)
        self.endtimeDateTimeEdit = QtGui.QDateTimeEdit(self.timeGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.endtimeDateTimeEdit.sizePolicy().hasHeightForWidth())
        self.endtimeDateTimeEdit.setSizePolicy(sizePolicy)
        self.endtimeDateTimeEdit.setProperty("showGroupSeparator", False)
        self.endtimeDateTimeEdit.setCalendarPopup(True)
        self.endtimeDateTimeEdit.setTimeSpec(QtCore.Qt.UTC)
        self.endtimeDateTimeEdit.setObjectName(_fromUtf8("endtimeDateTimeEdit"))
        self.formLayout_2.setWidget(1, QtGui.QFormLayout.FieldRole, self.endtimeDateTimeEdit)
        self.verticalLayout_6.addLayout(self.formLayout_2)
        self.horizontalLayout_7 = QtGui.QHBoxLayout()
        self.horizontalLayout_7.setObjectName(_fromUtf8("horizontalLayout_7"))
        self.time30DaysPushButton = QtGui.QPushButton(self.timeGroupBox)
        self.time30DaysPushButton.setObjectName(_fromUtf8("time30DaysPushButton"))
        self.horizontalLayout_7.addWidget(self.time30DaysPushButton)
        self.time1YearPushButton = QtGui.QPushButton(self.timeGroupBox)
        self.time1YearPushButton.setObjectName(_fromUtf8("time1YearPushButton"))
        self.horizontalLayout_7.addWidget(self.time1YearPushButton)
        self.verticalLayout_6.addLayout(self.horizontalLayout_7)
        self.verticalLayout_2.addLayout(self.verticalLayout_6)
        self.line_3 = QtGui.QFrame(self.timeGroupBox)
        self.line_3.setFrameShape(QtGui.QFrame.HLine)
        self.line_3.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_3.setObjectName(_fromUtf8("line_3"))
        self.verticalLayout_2.addWidget(self.line_3)
        self.timeFromEventsToolButton = QtGui.QToolButton(self.timeGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeFromEventsToolButton.sizePolicy().hasHeightForWidth())
        self.timeFromEventsToolButton.setSizePolicy(sizePolicy)
        self.timeFromEventsToolButton.setObjectName(_fromUtf8("timeFromEventsToolButton"))
        self.verticalLayout_2.addWidget(self.timeFromEventsToolButton)
        self.verticalLayout.addWidget(self.timeGroupBox)
        self.locationGroupBox = QtGui.QGroupBox(StationOptionsWidget)
        self.locationGroupBox.setObjectName(_fromUtf8("locationGroupBox"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.locationGroupBox)
        self.verticalLayout_4.setContentsMargins(-1, -1, -1, 4)
        self.verticalLayout_4.setSpacing(4)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.locationGlobalRadioButton = QtGui.QRadioButton(self.locationGroupBox)
        self.locationGlobalRadioButton.setChecked(False)
        self.locationGlobalRadioButton.setObjectName(_fromUtf8("locationGlobalRadioButton"))
        self.horizontalLayout_3.addWidget(self.locationGlobalRadioButton)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.line_4 = QtGui.QFrame(self.locationGroupBox)
        self.line_4.setFrameShape(QtGui.QFrame.HLine)
        self.line_4.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_4.setObjectName(_fromUtf8("line_4"))
        self.verticalLayout_4.addWidget(self.line_4)
        self.verticalLayout_8 = QtGui.QVBoxLayout()
        self.verticalLayout_8.setSpacing(0)
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        self.horizontalLayout_8 = QtGui.QHBoxLayout()
        self.horizontalLayout_8.setSpacing(0)
        self.horizontalLayout_8.setObjectName(_fromUtf8("horizontalLayout_8"))
        self.locationRangeRadioButton = QtGui.QRadioButton(self.locationGroupBox)
        self.locationRangeRadioButton.setChecked(False)
        self.locationRangeRadioButton.setObjectName(_fromUtf8("locationRangeRadioButton"))
        self.horizontalLayout_8.addWidget(self.locationRangeRadioButton)
        spacerItem = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem)
        self.drawLocationRangeToolButton = QtGui.QToolButton(self.locationGroupBox)
        self.drawLocationRangeToolButton.setCheckable(True)
        self.drawLocationRangeToolButton.setObjectName(_fromUtf8("drawLocationRangeToolButton"))
        self.horizontalLayout_8.addWidget(self.drawLocationRangeToolButton)
        self.verticalLayout_8.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(24, -1, -1, -1)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.verticalLayout_7 = QtGui.QVBoxLayout()
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setObjectName(_fromUtf8("verticalLayout_7"))
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setSpacing(0)
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        spacerItem1 = QtGui.QSpacerItem(1, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.locationRangeNorthDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.locationRangeNorthDoubleSpinBox.setMinimum(-90.0)
        self.locationRangeNorthDoubleSpinBox.setMaximum(90.0)
        self.locationRangeNorthDoubleSpinBox.setObjectName(_fromUtf8("locationRangeNorthDoubleSpinBox"))
        self.horizontalLayout_4.addWidget(self.locationRangeNorthDoubleSpinBox)
        spacerItem2 = QtGui.QSpacerItem(1, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.verticalLayout_7.addLayout(self.horizontalLayout_4)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.locationRangeWestDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.locationRangeWestDoubleSpinBox.setMinimum(-180.0)
        self.locationRangeWestDoubleSpinBox.setMaximum(180.0)
        self.locationRangeWestDoubleSpinBox.setObjectName(_fromUtf8("locationRangeWestDoubleSpinBox"))
        self.horizontalLayout.addWidget(self.locationRangeWestDoubleSpinBox)
        self.locationRangeEastDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.locationRangeEastDoubleSpinBox.setMinimum(-180.0)
        self.locationRangeEastDoubleSpinBox.setMaximum(180.0)
        self.locationRangeEastDoubleSpinBox.setObjectName(_fromUtf8("locationRangeEastDoubleSpinBox"))
        self.horizontalLayout.addWidget(self.locationRangeEastDoubleSpinBox)
        self.verticalLayout_7.addLayout(self.horizontalLayout)
        self.horizontalLayout_5 = QtGui.QHBoxLayout()
        self.horizontalLayout_5.setSpacing(0)
        self.horizontalLayout_5.setObjectName(_fromUtf8("horizontalLayout_5"))
        spacerItem3 = QtGui.QSpacerItem(1, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem3)
        self.locationRangeSouthDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.locationRangeSouthDoubleSpinBox.setMinimum(-90.0)
        self.locationRangeSouthDoubleSpinBox.setMaximum(90.0)
        self.locationRangeSouthDoubleSpinBox.setObjectName(_fromUtf8("locationRangeSouthDoubleSpinBox"))
        self.horizontalLayout_5.addWidget(self.locationRangeSouthDoubleSpinBox)
        spacerItem4 = QtGui.QSpacerItem(1, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem4)
        self.verticalLayout_7.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_2.addLayout(self.verticalLayout_7)
        spacerItem5 = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem5)
        self.horizontalLayout_2.setStretch(1, 1)
        self.verticalLayout_8.addLayout(self.horizontalLayout_2)
        self.verticalLayout_4.addLayout(self.verticalLayout_8)
        self.line_2 = QtGui.QFrame(self.locationGroupBox)
        self.line_2.setFrameShape(QtGui.QFrame.HLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName(_fromUtf8("line_2"))
        self.verticalLayout_4.addWidget(self.line_2)
        self.verticalLayout_9 = QtGui.QVBoxLayout()
        self.verticalLayout_9.setSpacing(0)
        self.verticalLayout_9.setObjectName(_fromUtf8("verticalLayout_9"))
        self.horizontalLayout_9 = QtGui.QHBoxLayout()
        self.horizontalLayout_9.setSpacing(0)
        self.horizontalLayout_9.setObjectName(_fromUtf8("horizontalLayout_9"))
        self.locationDistanceFromPointRadioButton = QtGui.QRadioButton(self.locationGroupBox)
        self.locationDistanceFromPointRadioButton.setEnabled(True)
        self.locationDistanceFromPointRadioButton.setObjectName(_fromUtf8("locationDistanceFromPointRadioButton"))
        self.horizontalLayout_9.addWidget(self.locationDistanceFromPointRadioButton)
        spacerItem6 = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem6)
        self.drawDistanceFromPointToolButton = QtGui.QToolButton(self.locationGroupBox)
        self.drawDistanceFromPointToolButton.setCheckable(True)
        self.drawDistanceFromPointToolButton.setObjectName(_fromUtf8("drawDistanceFromPointToolButton"))
        self.horizontalLayout_9.addWidget(self.drawDistanceFromPointToolButton)
        self.verticalLayout_9.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setContentsMargins(24, -1, -1, -1)
        self.horizontalLayout_6.setSpacing(0)
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        self.verticalLayout_3 = QtGui.QVBoxLayout()
        self.verticalLayout_3.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.verticalLayout_3.setSpacing(1)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.horizontalLayout_14 = QtGui.QHBoxLayout()
        self.horizontalLayout_14.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.horizontalLayout_14.setSpacing(4)
        self.horizontalLayout_14.setObjectName(_fromUtf8("horizontalLayout_14"))
        self.distanceFromPointMinRadiusDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.distanceFromPointMinRadiusDoubleSpinBox.setMinimum(0.0)
        self.distanceFromPointMinRadiusDoubleSpinBox.setMaximum(180.0)
        self.distanceFromPointMinRadiusDoubleSpinBox.setObjectName(_fromUtf8("distanceFromPointMinRadiusDoubleSpinBox"))
        self.horizontalLayout_14.addWidget(self.distanceFromPointMinRadiusDoubleSpinBox)
        self.label_2 = QtGui.QLabel(self.locationGroupBox)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.horizontalLayout_14.addWidget(self.label_2)
        self.distanceFromPointMaxRadiusDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.distanceFromPointMaxRadiusDoubleSpinBox.setMinimum(0.0)
        self.distanceFromPointMaxRadiusDoubleSpinBox.setMaximum(180.0)
        self.distanceFromPointMaxRadiusDoubleSpinBox.setObjectName(_fromUtf8("distanceFromPointMaxRadiusDoubleSpinBox"))
        self.horizontalLayout_14.addWidget(self.distanceFromPointMaxRadiusDoubleSpinBox)
        self.label_8 = QtGui.QLabel(self.locationGroupBox)
        self.label_8.setEnabled(True)
        self.label_8.setAlignment(QtCore.Qt.AlignCenter)
        self.label_8.setObjectName(_fromUtf8("label_8"))
        self.horizontalLayout_14.addWidget(self.label_8)
        self.verticalLayout_3.addLayout(self.horizontalLayout_14)
        self.horizontalLayout_15 = QtGui.QHBoxLayout()
        self.horizontalLayout_15.setSpacing(4)
        self.horizontalLayout_15.setObjectName(_fromUtf8("horizontalLayout_15"))
        self.label_6 = QtGui.QLabel(self.locationGroupBox)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.horizontalLayout_15.addWidget(self.label_6)
        self.distanceFromPointEastDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.distanceFromPointEastDoubleSpinBox.setMinimum(-180.0)
        self.distanceFromPointEastDoubleSpinBox.setMaximum(180.0)
        self.distanceFromPointEastDoubleSpinBox.setObjectName(_fromUtf8("distanceFromPointEastDoubleSpinBox"))
        self.horizontalLayout_15.addWidget(self.distanceFromPointEastDoubleSpinBox)
        self.label_14 = QtGui.QLabel(self.locationGroupBox)
        self.label_14.setEnabled(True)
        self.label_14.setObjectName(_fromUtf8("label_14"))
        self.horizontalLayout_15.addWidget(self.label_14)
        self.distanceFromPointNorthDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.distanceFromPointNorthDoubleSpinBox.setMinimum(-90.0)
        self.distanceFromPointNorthDoubleSpinBox.setMaximum(90.0)
        self.distanceFromPointNorthDoubleSpinBox.setObjectName(_fromUtf8("distanceFromPointNorthDoubleSpinBox"))
        self.horizontalLayout_15.addWidget(self.distanceFromPointNorthDoubleSpinBox)
        self.label_7 = QtGui.QLabel(self.locationGroupBox)
        self.label_7.setObjectName(_fromUtf8("label_7"))
        self.horizontalLayout_15.addWidget(self.label_7)
        self.verticalLayout_3.addLayout(self.horizontalLayout_15)
        self.horizontalLayout_6.addLayout(self.verticalLayout_3)
        spacerItem7 = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem7)
        self.horizontalLayout_6.setStretch(1, 1)
        self.verticalLayout_9.addLayout(self.horizontalLayout_6)
        self.verticalLayout_4.addLayout(self.verticalLayout_9)
        self.line = QtGui.QFrame(self.locationGroupBox)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName(_fromUtf8("line"))
        self.verticalLayout_4.addWidget(self.line)
        self.verticalLayout_10 = QtGui.QVBoxLayout()
        self.verticalLayout_10.setSpacing(0)
        self.verticalLayout_10.setObjectName(_fromUtf8("verticalLayout_10"))
        self.horizontalLayout_10 = QtGui.QHBoxLayout()
        self.horizontalLayout_10.setSpacing(0)
        self.horizontalLayout_10.setObjectName(_fromUtf8("horizontalLayout_10"))
        self.locationDistanceFromEventsRadioButton = QtGui.QRadioButton(self.locationGroupBox)
        self.locationDistanceFromEventsRadioButton.setEnabled(True)
        self.locationDistanceFromEventsRadioButton.setObjectName(_fromUtf8("locationDistanceFromEventsRadioButton"))
        self.horizontalLayout_10.addWidget(self.locationDistanceFromEventsRadioButton)
        self.verticalLayout_10.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_11 = QtGui.QHBoxLayout()
        self.horizontalLayout_11.setContentsMargins(24, -1, -1, -1)
        self.horizontalLayout_11.setSpacing(0)
        self.horizontalLayout_11.setObjectName(_fromUtf8("horizontalLayout_11"))
        self.verticalLayout_11 = QtGui.QVBoxLayout()
        self.verticalLayout_11.setSpacing(1)
        self.verticalLayout_11.setObjectName(_fromUtf8("verticalLayout_11"))
        self.horizontalLayout_16 = QtGui.QHBoxLayout()
        self.horizontalLayout_16.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.horizontalLayout_16.setSpacing(4)
        self.horizontalLayout_16.setObjectName(_fromUtf8("horizontalLayout_16"))
        self.distanceFromEventsMinDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.distanceFromEventsMinDoubleSpinBox.setMinimum(0.0)
        self.distanceFromEventsMinDoubleSpinBox.setMaximum(180.0)
        self.distanceFromEventsMinDoubleSpinBox.setObjectName(_fromUtf8("distanceFromEventsMinDoubleSpinBox"))
        self.horizontalLayout_16.addWidget(self.distanceFromEventsMinDoubleSpinBox)
        self.label_3 = QtGui.QLabel(self.locationGroupBox)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.horizontalLayout_16.addWidget(self.label_3)
        self.distanceFromEventsMaxDoubleSpinBox = QtGui.QDoubleSpinBox(self.locationGroupBox)
        self.distanceFromEventsMaxDoubleSpinBox.setMinimum(0.0)
        self.distanceFromEventsMaxDoubleSpinBox.setMaximum(180.0)
        self.distanceFromEventsMaxDoubleSpinBox.setObjectName(_fromUtf8("distanceFromEventsMaxDoubleSpinBox"))
        self.horizontalLayout_16.addWidget(self.distanceFromEventsMaxDoubleSpinBox)
        self.label_9 = QtGui.QLabel(self.locationGroupBox)
        self.label_9.setEnabled(True)
        self.label_9.setAlignment(QtCore.Qt.AlignCenter)
        self.label_9.setObjectName(_fromUtf8("label_9"))
        self.horizontalLayout_16.addWidget(self.label_9)
        self.verticalLayout_11.addLayout(self.horizontalLayout_16)
        self.horizontalLayout_11.addLayout(self.verticalLayout_11)
        spacerItem8 = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem8)
        self.horizontalLayout_11.setStretch(1, 1)
        self.verticalLayout_10.addLayout(self.horizontalLayout_11)
        self.verticalLayout_4.addLayout(self.verticalLayout_10)
        self.line_5 = QtGui.QFrame(self.locationGroupBox)
        self.line_5.setFrameShape(QtGui.QFrame.HLine)
        self.line_5.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_5.setObjectName(_fromUtf8("line_5"))
        self.verticalLayout_4.addWidget(self.line_5)
        self.locationFromEventsToolButton = QtGui.QToolButton(self.locationGroupBox)
        self.locationFromEventsToolButton.setEnabled(True)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.locationFromEventsToolButton.sizePolicy().hasHeightForWidth())
        self.locationFromEventsToolButton.setSizePolicy(sizePolicy)
        self.locationFromEventsToolButton.setObjectName(_fromUtf8("locationFromEventsToolButton"))
        self.verticalLayout_4.addWidget(self.locationFromEventsToolButton)
        self.verticalLayout.addWidget(self.locationGroupBox)
        spacerItem9 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem9)

        self.retranslateUi(StationOptionsWidget)
        QtCore.QMetaObject.connectSlotsByName(StationOptionsWidget)
        StationOptionsWidget.setTabOrder(self.networkLineEdit, self.stationLineEdit)
        StationOptionsWidget.setTabOrder(self.stationLineEdit, self.locationLineEdit)
        StationOptionsWidget.setTabOrder(self.locationLineEdit, self.channelLineEdit)
        StationOptionsWidget.setTabOrder(self.channelLineEdit, self.starttimeDateTimeEdit)
        StationOptionsWidget.setTabOrder(self.starttimeDateTimeEdit, self.endtimeDateTimeEdit)
        StationOptionsWidget.setTabOrder(self.endtimeDateTimeEdit, self.timeFromEventsToolButton)
        StationOptionsWidget.setTabOrder(self.timeFromEventsToolButton, self.locationFromEventsToolButton)

    def retranslateUi(self, StationOptionsWidget):
        self.SNCLGroupBox.setTitle(_translate("StationOptionsWidget", "SNCL", None))
        self.networkLabel.setText(_translate("StationOptionsWidget", "Networks", None))
        self.networkLineEdit.setText(_translate("StationOptionsWidget", "_GSN", None))
        self.stationLineEdit.setText(_translate("StationOptionsWidget", "*", None))
        self.locationLabel.setText(_translate("StationOptionsWidget", "Locations", None))
        self.locationLineEdit.setText(_translate("StationOptionsWidget", "*", None))
        self.channelLabel.setText(_translate("StationOptionsWidget", "Channels", None))
        self.channelLineEdit.setText(_translate("StationOptionsWidget", "?HZ", None))
        self.stationLabel.setText(_translate("StationOptionsWidget", "Stations", None))
        self.timeGroupBox.setTitle(_translate("StationOptionsWidget", "Time", None))
        self.starttimeLabel.setText(_translate("StationOptionsWidget", "Start", None))
        self.starttimeDateTimeEdit.setDisplayFormat(_translate("StationOptionsWidget", "yyyy-MM-dd hh:mm:ss", None))
        self.endtimeLabel.setText(_translate("StationOptionsWidget", "End", None))
        self.endtimeDateTimeEdit.setDisplayFormat(_translate("StationOptionsWidget", "yyyy-MM-dd hh:mm:ss", None))
        self.time30DaysPushButton.setText(_translate("StationOptionsWidget", "Last 30 days", None))
        self.time1YearPushButton.setText(_translate("StationOptionsWidget", "Last year", None))
        self.timeFromEventsToolButton.setText(_translate("StationOptionsWidget", ">> Copy Time Range from Event Options", None))
        self.locationGroupBox.setTitle(_translate("StationOptionsWidget", "Location", None))
        self.locationGlobalRadioButton.setText(_translate("StationOptionsWidget", "Global", None))
        self.locationRangeRadioButton.setText(_translate("StationOptionsWidget", "Within lat/lon box", None))
        self.drawLocationRangeToolButton.setText(_translate("StationOptionsWidget", "Draw on map", None))
        self.locationDistanceFromPointRadioButton.setText(_translate("StationOptionsWidget", "Distance from point", None))
        self.drawDistanceFromPointToolButton.setText(_translate("StationOptionsWidget", "Draw on map", None))
        self.label_2.setText(_translate("StationOptionsWidget", "-", None))
        self.label_8.setText(_translate("StationOptionsWidget", "degrees", None))
        self.label_6.setText(_translate("StationOptionsWidget", "from", None))
        self.label_14.setText(_translate("StationOptionsWidget", "E", None))
        self.label_7.setText(_translate("StationOptionsWidget", "N", None))
        self.locationDistanceFromEventsRadioButton.setText(_translate("StationOptionsWidget", "Distance from selected events", None))
        self.label_3.setText(_translate("StationOptionsWidget", "-", None))
        self.label_9.setText(_translate("StationOptionsWidget", "degrees", None))
        self.locationFromEventsToolButton.setText(_translate("StationOptionsWidget", ">> Copy Location from Event Options", None))




# -*- coding: utf-8 -*-
"""
Dialog containing an IPython console.

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from qtconsole.rich_ipython_widget import RichIPythonWidget
from qtconsole.inprocess import QtInProcessKernelManager
from PyQt4 import QtGui
from pyweed.gui.BaseDialog import BaseDialog


class EmbedIPython(RichIPythonWidget):

    def __init__(self, **kwarg):
        super(RichIPythonWidget, self).__init__()
        self.kernel_manager = QtInProcessKernelManager()
        self.kernel_manager.start_kernel()
        self.kernel = self.kernel_manager.kernel
        self.kernel.gui = 'qt4'
        self.kernel.shell.push(kwarg)
        self.kernel_client = self.kernel_manager.client()
        self.kernel_client.start_channels()


class ConsoleDialog(BaseDialog):
    def __init__(self, pyweed, parent=None):
        super(ConsoleDialog, self).__init__(parent=parent)
        self.widget = EmbedIPython(pyweed=pyweed)
        layout = QtGui.QVBoxLayout(self)
        layout.addWidget(self.widget)


if __name__ == '__main__':
    import sys
    app = QtGui.QApplication(sys.argv)
    widget = ConsoleDialog()
    widget.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""
Application preferences

Adapted from: https://github.com/claysmith/oldArcD/blob/master/tools/arctographer/arcmap/preferences.py

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from __future__ import (absolute_import, division, print_function)

import os
import platform
from future.moves.configparser import SafeConfigParser


def safe_bool(s, default=False):
    try:
        return s.startswith('y')
    except:
        return default


def bool_to_str(b):
    if b:
        return "y"
    else:
        return "n"


def safe_int(s, default=0):
    try:
        return int(s)
    except:
        return default


class Section(object):
    def __init__(self, **initial):
        self.__dict__.update(initial)

    def read_from_config(self, config):
        name = self.__name__
        if config.has_section(name):
            fix_case = dict((k.lower(), k) for k in self.__dict__.keys())
            for k, v in config.items(name):
                if k in fix_case:
                    k = fix_case[k]
                self.__dict__[k] = v

    def write_to_config(self, config):
        name = self.__name__
        config.add_section(name)
        for key, value in self.items():
            config.set(name, key, str(value))

    @property
    def __name__(self):
        return self.__class__.__name__

    def items(self):
        return self.__dict__.items()

    def update(self, values):
        self.__dict__.update(values)

    def __repr__(self):
        return self.__dict__.__repr__()

    @classmethod
    def create(cls, section_name, **initial):
        return type(section_name, (cls,), {})(**initial)


class Preferences(object):
    """
    Container for application preferences.
    """

    def __init__(self):
        """
        Initialization with default settings.
        """

#         for (section, prefs) in DEFAULTS.items():
#             setattr(self, section, Section.create(section, **prefs))

        self.Data = Section.create("Data")
        self.Data.eventDataCenter = "IRIS"
        self.Data.stationDataCenter = "IRIS"

        self.Waveforms = Section.create("Waveforms")
        self.Waveforms.downloadDir = user_download_path()
        self.Waveforms.cacheSize = "50"  # megabytes
        self.Waveforms.saveDir = user_save_path()
        self.Waveforms.timeWindowBefore = "60"  # seconds
        self.Waveforms.timeWindowBeforePhase = "P"  # P|S|Event
        self.Waveforms.timeWindowAfter = "600"  # seconds
        self.Waveforms.timeWindowAfterPhase = "P"  # P|S|Event
        self.Waveforms.saveFormat = "MSEED"

        self.Logging = Section.create("Logging")
        self.Logging.level = "INFO"

        self.Map = Section.create("Map")

        self.MainWindow = Section.create("MainWindow")
        self.MainWindow.width = "1000"
        self.MainWindow.height = "800"
        self.MainWindow.eventOptionsFloat = "n"
        self.MainWindow.stationOptionsFloat = "n"

        self.EventOptions = Section.create("EventOptions")

        self.StationOptions = Section.create("StationOptions")

    def save(self):
        """
        Saves the user's preferences to config file
        """
        config = SafeConfigParser()

        sections = [
            self.Data,
            self.Waveforms,
            self.Logging,
            self.Map,
            self.MainWindow,
            self.EventOptions,
            self.StationOptions,
        ]
        for section in sections:
            section.write_to_config(config)

        if not os.path.exists(user_config_path()):
            try:
                os.makedirs(user_config_path(), 0o700)
            except Exception as e:
                print("Creation of user configuration directory failed with" + " error: \"%s\'""" % e)
                return
        f = open(os.path.join(user_config_path(), "pyweed.ini"), "w")
        config.write(f)

    def load(self):
        """
        Loads the user's preferences from saved config file
        '"""
        path = os.path.join(user_config_path(), "pyweed.ini")

        if not os.path.exists(path):
            # Save the default configuration info
            self.save()

        else:
            # Override defaults with anything found in config.ini
            f = open(path, "r")
            config = SafeConfigParser()
            config.readfp(f)

            sections = [
                self.Data,
                self.Waveforms,
                self.Logging,
                self.Map,
                self.MainWindow,
                self.EventOptions,
                self.StationOptions,
            ]
            for section in sections:
                section.read_from_config(config)


# ------------------------------------------------------------------------------
# Helper functions
# ------------------------------------------------------------------------------

def user_config_path(safe=True):
    """
    @param safe: If set, auto-create the path if it doesn't exist
    @rtype: str
    @return: the directory for storing user configuration files
    """
    p = os.path.join(os.path.expanduser("~"), ".pyweed")
    if safe:
        os.makedirs(p, exist_ok=True)
    return p
#     if platform.system() == "Darwin":
#         return os.path.join(os.path.expanduser("~"), "Library", "Preferences")
#     elif platform.system() == "Windows":
#         return os.path.join(os.path.expanduser("~"), "AppData", "Local")
#     else:
#         # Assume a Linux-like system
#         return os.path.join(os.path.expanduser("~"), ".config")


def user_download_path(safe=True):
    """
    @param safe: If set, auto-create the path if it doesn't exist
    @rtype: str
    @return: the default directory for saving downloaded (previewed) waveform data
    """
    p = os.path.join(os.path.expanduser("~"), ".pyweed", "data")
    if safe:
        os.makedirs(p, exist_ok=True)
    return p
#     if platform.system() == "Darwin":
#         return os.path.join(os.path.expanduser("~"), "Library", "Application Support", "pyweed_data")
#     elif platform.system() == "Windows":
#         return os.path.join(os.path.expanduser("~"), "AppData", "Local", "pyweed_data")
#     else:
#         # Assume a Linux-like system
#         return os.path.join(os.path.expanduser("~"), ".config", "pyweed_data")


def user_save_path(safe=False):
    """
    @param safe: If set, auto-create the path if it doesn't exist
    @rtype: str
    @return: the default directory for saving waveform files
    """
    p = os.path.join(os.path.expanduser("~"), "Downloads", "pyweed")
    if safe:
        os.makedirs(p, exist_ok=True)
    return p
#     if platform.system() == "Darwin":
#         return os.path.join(os.path.expanduser("~"), "Downloads", "pyweed")
#     elif platform.system() == "Windows":
#         return os.path.join(os.path.expanduser("~"), "Documents", "pyweed")
#     else:
#         # Assume a Linux-like system
#         return os.path.join(os.path.expanduser("~"), "Downloads", "pyweed")


# ------------------------------------------------------------------------------
# Main
# ------------------------------------------------------------------------------

if __name__ == '__main__':
    import doctest
    doctest.testmod(exclude_empty=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PyWEED GUI Launcher

This exists mainly to show the splash screen as quickly as possible, by deferring lots of the expensive
library imports required for the application itself.

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from __future__ import (absolute_import, division, print_function)

import sys
import os
from future.utils import PY2
import matplotlib

# If running under windows, redirect stdout/stderr, since writing to them will crash Python if there's
# not a console. See https://bugs.python.org/issue706263
if sys.executable.endswith('pythonw.exe'):
    nul = open(os.devnull, 'w')
    sys.stderr = nul
    sys.stdout = nul

# Set up the PROJ_LIB environment variable needed by matplotlib
# See https://github.com/conda-forge/basemap-feedstock/issues/30
if not os.environ.get('PROJ_LIB') and os.environ.get('CONDA_PREFIX'):
    os.environ['PROJ_LIB'] = os.path.join(os.environ['CONDA_PREFIX'], 'share', 'proj')

# Configure matplotlib backend
matplotlib.use('AGG')

# For debugging, raise an exception on attempted chained assignment
# See http://pandas.pydata.org/pandas-docs/version/0.19.1/indexing.html#returning-a-view-versus-a-copy
# import pandas as pd
# pd.set_option('mode.chained_assignment', 'raise')

if PY2:
    # Configure PyQt4 -- in order for the Python console to work in Python 2, we need to load a particular
    # version of some internal libraries. This must be done before the first import of the PyQt4 libraries.
    # See http://stackoverflow.com/questions/11513132/embedding-ipython-qt-console-in-a-pyqt-application/20610786#20610786
    os.environ['QT_API'] = 'pyqt'
    import sip
    sip.setapi("QString", 2)
    sip.setapi("QVariant", 2)


def init_strptime():
    """ Workaround for https://github.com/obspy/obspy/issues/2147 """
    try:
        import locale
        # locale.setlocale(locale.LC_TIME, ('en_US', 'UTF-8'))
        from obspy.clients.fdsn.routing.routing_client import RoutingClient  # NOQA
        locale.setlocale(locale.LC_TIME, '')
    except Exception:
        pass
init_strptime()


def get_pyweed():
    """
    Load the PyWEED GUI code, this is where most of the expensive stuff happens
    """
    from pyweed.gui.PyWeedGUI import PyWeedGUI
    return PyWeedGUI()


def launch():
    """
    Basic startup process.
    """
    from PyQt4 import QtGui, QtCore
    from pyweed.gui.SplashScreenHandler import SplashScreenHandler
    import pyweed.gui.qrc  # NOQA

    # See https://stackoverflow.com/questions/31952711/
    QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_X11InitThreads)

    app = QtGui.QApplication(sys.argv)
    splashScreenHandler = SplashScreenHandler()
    app.processEvents()
    pyweed = get_pyweed()
    splashScreenHandler.finish(pyweed.mainWindow)
    sys.exit(app.exec_())


if __name__ == "__main__":
    launch()



# -*- coding: utf-8 -*-
"""
Selecting stations based on selected events.

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from __future__ import (absolute_import, division, print_function)
from logging import getLogger

LOGGER = getLogger(__name__)


class CrossBorderException(Exception):
    pass


class LatLonBox(object):
    lat1 = None
    lat2 = None
    lon1 = None
    lon2 = None

    def __init__(self, distance):
        self.distance = distance

    def add_point(self, lat, lon):
        if self.lat1 is None:
            self.lat1 = lat - self.distance
            self.lat2 = lat + self.distance
            self.lon1 = lon - self.distance
            self.lon2 = lon + self.distance
        else:
            self.lat1 = min(self.lat1, lat - self.distance)
            self.lat2 = max(self.lat2, lat + self.distance)
            self.lon1 = min(self.lon1, lon - self.distance)
            self.lon2 = max(self.lon2, lon + self.distance)

        if self.lat1 < -90 or self.lat2 > 90 or self.lon1 < -180 or self.lon2 > 180:
            raise CrossBorderException()

    def __str__(self):
        return "[%s, %s, %s, %s]" % (self.lat1, self.lon1, self.lat2, self.lon2)


def get_combined_locations(points, distance):
    """
    Given a set of points and a distance, return a reasonable set of
    bounding areas covering everything within that distance of any point.
    """
    # Very simple implementation, try to draw a box around all points
    box = LatLonBox(distance)

    if not points:
        return []

    for lat, lon in points:
        box.add_point(lat, lon)
    LOGGER.info("Combined points into: %s", box)
    return [box]



# -*- coding: utf-8 -*-
"""
Manage the splash screen

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

import sys
import logging
from PyQt4 import QtGui, QtCore


class SplashScreenHandler(logging.Handler):

    def __init__(self,):
        super(SplashScreenHandler, self).__init__(level=logging.INFO)
        pixmap = QtGui.QPixmap(":qrc/splash.png")
        self.splash = QtGui.QSplashScreen(pixmap, QtCore.Qt.WindowStaysOnTopHint)

        # Attach as handler to the root logger
        logger = logging.getLogger()
        logger.addHandler(self)

        self.splash.show()
        logger.info("Splash screen should be visible")

    def emit(self, record):
        msg = self.format(record)
        self.splash.showMessage(msg)
        QtGui.QApplication.processEvents()

    def finish(self, mainWin):
        super(SplashScreenHandler, self).close()
        self.splash.finish(mainWin)
        logger = logging.getLogger()
        logger.removeHandler(self)
        logger.info("Splash screen finished")



# -*- coding: utf-8 -*-
"""
Base class for the *Dialog widgets.
This is necessary because there are some significant differences among platforms, which we want to
normalize for all of our secondary windows.

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from PyQt4 import QtGui, QtCore
import platform

# Identify the platform
IS_DARWIN = (platform.system() == 'Darwin')
IS_LINUX = (platform.system() == 'Linux')


class BaseDialog(QtGui.QDialog):
    def __init__(self, parent=None, *args, **kwargs):
        # On non-Mac platforms, a dialog with a parent will always float above the parent. We want these
        # windows to be independent, so in that case remove the parent.
        # (We need the parent on Mac because it allows all windows to share a menu.)
        if not IS_DARWIN:
            parent = None
        super(BaseDialog, self).__init__(parent=parent, *args, **kwargs)
        # On Linux, dialogs don't have window controls, this can be fixed by turning off that window flag
        if IS_LINUX:
            self.setWindowFlags(self.windowFlags() & ~QtCore.Qt.Dialog)



# -*- coding: utf-8 -*-
"""
Various Qt widget utilities

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from PyQt4 import QtGui, QtCore
from logging import getLogger
from PyQt4.QtCore import pyqtSlot

LOGGER = getLogger(__name__)


class ComboBoxAdapter(QtCore.QObject):
    """
    Adapter for handling the fact that QComboBox can only display strings, but usually the "real" value
    is something else.
    """

    #: Signal emitted when the QComboBox value changes, passing the actual value
    changed = QtCore.pyqtSignal(object)

    def __init__(self, comboBox, options, default=None):
        """
        :param comboBox: A QComboBox
        :param options: A list of options in the form of (value, label)
        :param default: If nothing is selected, what should be returned?
        """
        super(ComboBoxAdapter, self).__init__()
        self.comboBox = comboBox
        self.values = [option[0] for option in options]
        if default is None:
            default = self.values[0]
        self.default = default
        labels = [option[1] for option in options]
        self.comboBox.addItems(labels)
        self.comboBox.currentIndexChanged.connect(self.onChanged)

    def setValue(self, value):
        self.comboBox.setCurrentIndex(self.values.index(value))

    def getValue(self, index=None):
        if index is None:
            index = self.comboBox.currentIndex()
        if index < 0:
            return self.default
        else:
            return self.values[index]

    @pyqtSlot(int)
    def onChanged(self, index):
        self.changed.emit(self.getValue(index))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pyweed/gui/uic/WaveformDialog.ui'
#
# Created by: PyQt4 UI code generator 4.11.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_WaveformDialog(object):
    def setupUi(self, WaveformDialog):
        WaveformDialog.setObjectName(_fromUtf8("WaveformDialog"))
        WaveformDialog.resize(1212, 724)
        self.verticalLayout = QtGui.QVBoxLayout(WaveformDialog)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.topFrame = QtGui.QFrame(WaveformDialog)
        self.topFrame.setFrameShape(QtGui.QFrame.NoFrame)
        self.topFrame.setFrameShadow(QtGui.QFrame.Plain)
        self.topFrame.setObjectName(_fromUtf8("topFrame"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.topFrame)
        self.horizontalLayout.setMargin(2)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.downloadGroupBox = QtGui.QGroupBox(self.topFrame)
        self.downloadGroupBox.setEnabled(True)
        self.downloadGroupBox.setObjectName(_fromUtf8("downloadGroupBox"))
        self.verticalLayout_6 = QtGui.QVBoxLayout(self.downloadGroupBox)
        self.verticalLayout_6.setMargin(2)
        self.verticalLayout_6.setSpacing(2)
        self.verticalLayout_6.setObjectName(_fromUtf8("verticalLayout_6"))
        self.gridLayout = QtGui.QGridLayout()
        self.gridLayout.setVerticalSpacing(2)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.secondsBeforeLabel = QtGui.QLabel(self.downloadGroupBox)
        self.secondsBeforeLabel.setObjectName(_fromUtf8("secondsBeforeLabel"))
        self.gridLayout.addWidget(self.secondsBeforeLabel, 0, 0, 1, 1)
        self.label = QtGui.QLabel(self.downloadGroupBox)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 2, 1, 1)
        self.secondsAfterPhaseComboBox = QtGui.QComboBox(self.downloadGroupBox)
        self.secondsAfterPhaseComboBox.setObjectName(_fromUtf8("secondsAfterPhaseComboBox"))
        self.gridLayout.addWidget(self.secondsAfterPhaseComboBox, 1, 3, 1, 1)
        self.secondsBeforePhaseComboBox = QtGui.QComboBox(self.downloadGroupBox)
        self.secondsBeforePhaseComboBox.setObjectName(_fromUtf8("secondsBeforePhaseComboBox"))
        self.gridLayout.addWidget(self.secondsBeforePhaseComboBox, 0, 3, 1, 1)
        spacerItem = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 4, 1, 1)
        self.secondsAfterSpinBox = QtGui.QSpinBox(self.downloadGroupBox)
        self.secondsAfterSpinBox.setMaximum(21600)
        self.secondsAfterSpinBox.setSingleStep(100)
        self.secondsAfterSpinBox.setProperty("value", 600)
        self.secondsAfterSpinBox.setObjectName(_fromUtf8("secondsAfterSpinBox"))
        self.gridLayout.addWidget(self.secondsAfterSpinBox, 1, 1, 1, 1)
        self.secondsBeforeSpinBox = QtGui.QSpinBox(self.downloadGroupBox)
        self.secondsBeforeSpinBox.setMaximum(3600)
        self.secondsBeforeSpinBox.setSingleStep(10)
        self.secondsBeforeSpinBox.setProperty("value", 60)
        self.secondsBeforeSpinBox.setObjectName(_fromUtf8("secondsBeforeSpinBox"))
        self.gridLayout.addWidget(self.secondsBeforeSpinBox, 0, 1, 1, 1)
        self.label_2 = QtGui.QLabel(self.downloadGroupBox)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 1, 2, 1, 1)
        self.secondsAfterLabel = QtGui.QLabel(self.downloadGroupBox)
        self.secondsAfterLabel.setObjectName(_fromUtf8("secondsAfterLabel"))
        self.gridLayout.addWidget(self.secondsAfterLabel, 1, 0, 1, 1)
        self.verticalLayout_6.addLayout(self.gridLayout)
        self.widget_5 = QtGui.QWidget(self.downloadGroupBox)
        self.widget_5.setObjectName(_fromUtf8("widget_5"))
        self.horizontalLayout_7 = QtGui.QHBoxLayout(self.widget_5)
        self.horizontalLayout_7.setMargin(2)
        self.horizontalLayout_7.setObjectName(_fromUtf8("horizontalLayout_7"))
        self.downloadPushButton = QtGui.QPushButton(self.widget_5)
        self.downloadPushButton.setMinimumSize(QtCore.QSize(200, 0))
        self.downloadPushButton.setCheckable(False)
        self.downloadPushButton.setObjectName(_fromUtf8("downloadPushButton"))
        self.horizontalLayout_7.addWidget(self.downloadPushButton)
        self.downloadStatusLabel = QtGui.QLabel(self.widget_5)
        self.downloadStatusLabel.setObjectName(_fromUtf8("downloadStatusLabel"))
        self.horizontalLayout_7.addWidget(self.downloadStatusLabel, QtCore.Qt.AlignLeft)
        self.verticalLayout_6.addWidget(self.widget_5)
        self.horizontalLayout.addWidget(self.downloadGroupBox)
        self.saveGroupBox = QtGui.QGroupBox(self.topFrame)
        self.saveGroupBox.setEnabled(True)
        self.saveGroupBox.setObjectName(_fromUtf8("saveGroupBox"))
        self.verticalLayout_5 = QtGui.QVBoxLayout(self.saveGroupBox)
        self.verticalLayout_5.setMargin(2)
        self.verticalLayout_5.setSpacing(2)
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.gridLayout_2 = QtGui.QGridLayout()
        self.gridLayout_2.setVerticalSpacing(2)
        self.gridLayout_2.setObjectName(_fromUtf8("gridLayout_2"))
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.saveFormatComboBox = QtGui.QComboBox(self.saveGroupBox)
        self.saveFormatComboBox.setObjectName(_fromUtf8("saveFormatComboBox"))
        self.horizontalLayout_4.addWidget(self.saveFormatComboBox)
        spacerItem1 = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.horizontalLayout_4.setStretch(1, 1)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 1, 1, 1, 1)
        self.saveDirectoryLabel = QtGui.QLabel(self.saveGroupBox)
        self.saveDirectoryLabel.setObjectName(_fromUtf8("saveDirectoryLabel"))
        self.gridLayout_2.addWidget(self.saveDirectoryLabel, 0, 0, 1, 1)
        self.saveFormatLabel = QtGui.QLabel(self.saveGroupBox)
        self.saveFormatLabel.setObjectName(_fromUtf8("saveFormatLabel"))
        self.gridLayout_2.addWidget(self.saveFormatLabel, 1, 0, 1, 1)
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        self.saveDirectoryPushButton = QtGui.QPushButton(self.saveGroupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.saveDirectoryPushButton.sizePolicy().hasHeightForWidth())
        self.saveDirectoryPushButton.setSizePolicy(sizePolicy)
        self.saveDirectoryPushButton.setFocusPolicy(QtCore.Qt.NoFocus)
        self.saveDirectoryPushButton.setStyleSheet(_fromUtf8("QPushButton { text-align: left; }"))
        self.saveDirectoryPushButton.setObjectName(_fromUtf8("saveDirectoryPushButton"))
        self.horizontalLayout_6.addWidget(self.saveDirectoryPushButton)
        self.saveDirectoryBrowseToolButton = QtGui.QToolButton(self.saveGroupBox)
        self.saveDirectoryBrowseToolButton.setIconSize(QtCore.QSize(16, 16))
        self.saveDirectoryBrowseToolButton.setObjectName(_fromUtf8("saveDirectoryBrowseToolButton"))
        self.horizontalLayout_6.addWidget(self.saveDirectoryBrowseToolButton)
        spacerItem2 = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem2)
        self.horizontalLayout_6.setStretch(2, 1)
        self.gridLayout_2.addLayout(self.horizontalLayout_6, 0, 1, 1, 1)
        self.verticalLayout_5.addLayout(self.gridLayout_2)
        self.widget_6 = QtGui.QWidget(self.saveGroupBox)
        self.widget_6.setObjectName(_fromUtf8("widget_6"))
        self.horizontalLayout_8 = QtGui.QHBoxLayout(self.widget_6)
        self.horizontalLayout_8.setMargin(2)
        self.horizontalLayout_8.setObjectName(_fromUtf8("horizontalLayout_8"))
        self.savePushButton = QtGui.QPushButton(self.widget_6)
        self.savePushButton.setMinimumSize(QtCore.QSize(200, 0))
        self.savePushButton.setCheckable(False)
        self.savePushButton.setObjectName(_fromUtf8("savePushButton"))
        self.horizontalLayout_8.addWidget(self.savePushButton)
        self.saveStatusLabel = QtGui.QLabel(self.widget_6)
        self.saveStatusLabel.setObjectName(_fromUtf8("saveStatusLabel"))
        self.horizontalLayout_8.addWidget(self.saveStatusLabel, QtCore.Qt.AlignLeft)
        self.verticalLayout_5.addWidget(self.widget_6)
        self.horizontalLayout.addWidget(self.saveGroupBox)
        self.verticalLayout.addWidget(self.topFrame)
        self.selectionTableFrame = QtGui.QFrame(WaveformDialog)
        self.selectionTableFrame.setObjectName(_fromUtf8("selectionTableFrame"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.selectionTableFrame)
        self.verticalLayout_2.setMargin(2)
        self.verticalLayout_2.setSpacing(2)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.filterGroupBox = QtGui.QGroupBox(self.selectionTableFrame)
        self.filterGroupBox.setEnabled(True)
        self.filterGroupBox.setObjectName(_fromUtf8("filterGroupBox"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout(self.filterGroupBox)
        self.horizontalLayout_2.setMargin(2)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.eventComboBox = QtGui.QComboBox(self.filterGroupBox)
        self.eventComboBox.setObjectName(_fromUtf8("eventComboBox"))
        self.horizontalLayout_2.addWidget(self.eventComboBox)
        self.networkComboBox = QtGui.QComboBox(self.filterGroupBox)
        self.networkComboBox.setObjectName(_fromUtf8("networkComboBox"))
        self.horizontalLayout_2.addWidget(self.networkComboBox)
        self.stationComboBox = QtGui.QComboBox(self.filterGroupBox)
        self.stationComboBox.setObjectName(_fromUtf8("stationComboBox"))
        self.horizontalLayout_2.addWidget(self.stationComboBox)
        self.horizontalLayout_2.setStretch(0, 3)
        self.horizontalLayout_2.setStretch(1, 1)
        self.horizontalLayout_2.setStretch(2, 1)
        self.verticalLayout_2.addWidget(self.filterGroupBox)
        self.selectionTable = QtGui.QTableWidget(self.selectionTableFrame)
        self.selectionTable.setMinimumSize(QtCore.QSize(500, 100))
        self.selectionTable.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.selectionTable.setSelectionMode(QtGui.QAbstractItemView.NoSelection)
        self.selectionTable.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.selectionTable.setVerticalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
        self.selectionTable.setHorizontalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
        self.selectionTable.setShowGrid(False)
        self.selectionTable.setObjectName(_fromUtf8("selectionTable"))
        self.selectionTable.setColumnCount(0)
        self.selectionTable.setRowCount(0)
        self.selectionTable.horizontalHeader().setStretchLastSection(False)
        self.selectionTable.verticalHeader().setVisible(True)
        self.verticalLayout_2.addWidget(self.selectionTable)
        self.verticalLayout.addWidget(self.selectionTableFrame)
        self.verticalLayout.setStretch(1, 1)

        self.retranslateUi(WaveformDialog)
        QtCore.QMetaObject.connectSlotsByName(WaveformDialog)

    def retranslateUi(self, WaveformDialog):
        WaveformDialog.setWindowTitle(_translate("WaveformDialog", "Dialog", None))
        self.downloadGroupBox.setTitle(_translate("WaveformDialog", "Download / Preview Waveforms", None))
        self.secondsBeforeLabel.setText(_translate("WaveformDialog", "Start:", None))
        self.label.setText(_translate("WaveformDialog", "secs before", None))
        self.label_2.setText(_translate("WaveformDialog", "secs after", None))
        self.secondsAfterLabel.setText(_translate("WaveformDialog", "End:", None))
        self.downloadPushButton.setText(_translate("WaveformDialog", "Download", None))
        self.downloadStatusLabel.setText(_translate("WaveformDialog", "Download status", None))
        self.saveGroupBox.setTitle(_translate("WaveformDialog", "Save Waveforms", None))
        self.saveDirectoryLabel.setText(_translate("WaveformDialog", "To:", None))
        self.saveFormatLabel.setText(_translate("WaveformDialog", "As:", None))
        self.saveDirectoryPushButton.setText(_translate("WaveformDialog", "Directory", None))
        self.saveDirectoryBrowseToolButton.setText(_translate("WaveformDialog", "Open", None))
        self.savePushButton.setText(_translate("WaveformDialog", "Save", None))
        self.saveStatusLabel.setText(_translate("WaveformDialog", "Save status", None))
        self.filterGroupBox.setTitle(_translate("WaveformDialog", "Table Filters", None))
        self.selectionTable.setSortingEnabled(True)




# -*- coding: utf-8 -*-
"""
Container for events.

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from __future__ import (absolute_import, division, print_function)

from pyweed.signals import SignalingThread, SignalingObject
import logging
from obspy.core.event.catalog import Catalog
from pyweed.pyweed_utils import get_service_url, CancelledException
from PyQt4 import QtCore
import concurrent.futures

LOGGER = logging.getLogger(__name__)


def load_events(client, parameters):
    """
    Execute one query for event data. This is a standalone function so we can
    run it in a separate thread.
    """
    try:
        LOGGER.info('Loading events: %s', get_service_url(client, 'event', parameters))
        return client.get_events(**parameters)
    except Exception as e:
        # If no results found, the client will raise an exception, we need to trap this
        # TODO: this should be much cleaner with a fix to https://github.com/obspy/obspy/issues/1656
        if str(e).startswith("No data"):
            LOGGER.warning("No events found! Your query may be too narrow.")
            return Catalog()
        else:
            raise


class EventsLoader(SignalingThread):
    """
    Thread to handle event requests
    """
    progress = QtCore.pyqtSignal()

    def __init__(self, request):
        """
        Initialization.
        """
        # Keep a reference to globally shared components
        self.request = request
        self.futures = {}
        super(EventsLoader, self).__init__()

    def run(self):
        """
        Make a webservice request for events using the passed in options.
        """
        self.setPriority(QtCore.QThread.LowestPriority)
        self.clearFutures()
        self.futures = {}

        catalog = None
        LOGGER.info("Making %d event requests", len(self.request.sub_requests))
        with concurrent.futures.ThreadPoolExecutor(5) as executor:
            for sub_request in self.request.sub_requests:
                # Dictionary lets us look up argument by result later
                self.futures[executor.submit(load_events, self.request.client, sub_request)] = sub_request
            # Iterate through Futures as they complete
            for result in concurrent.futures.as_completed(self.futures):
                LOGGER.debug("Events loaded")
                try:
                    if not catalog:
                        catalog = result.result()
                    else:
                        catalog += result.result()
                    self.progress.emit()
                except Exception:
                    self.progress.emit()
        self.futures = {}
        if not catalog:
            catalog = Catalog()
        LOGGER.info("Loader processing")
        catalog = self.request.process_result(catalog)
        LOGGER.info("Loader done")
        self.done.emit(catalog)

    def clearFutures(self):
        """
        Cancel any outstanding tasks
        """
        if self.futures:
            for future in self.futures:
                if not future.done():
                    LOGGER.debug("Cancelling unexecuted future")
                    future.cancel()

    def cancel(self):
        """
        User-requested cancel
        """
        self.done.disconnect()
        self.progress.disconnect()
        self.clearFutures()


class EventsHandler(SignalingObject):
    """
    Container for events.
    """

    def __init__(self, pyweed):
        """
        Initialization.
        """
        super(EventsHandler, self).__init__()
        self.pyweed = pyweed
        self.catalog_loader = None

    def load_catalog(self, request):
        self.catalog_loader = EventsLoader(request)
        self.catalog_loader.done.connect(self.on_catalog_loaded)
        self.catalog_loader.start()

    def on_catalog_loaded(self, event_catalog):
        self.done.emit(event_catalog)

    def cancel(self):
        if self.catalog_loader:
            self.catalog_loader.done.disconnect()
        self.done.emit(CancelledException())


# ------------------------------------------------------------------------------
# Main
# ------------------------------------------------------------------------------

if __name__ == '__main__':
    import doctest
    doctest.testmod(exclude_empty=True)



# -*- coding: utf-8 -*-
"""
Event Options

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from pyweed.options import Options, DateOption, FloatOption, Option


class EventOptions(Options):

    TIME_RANGE = 'timeBetween'
    TIME_STATIONS = 'timeFromStations'
    time_choice = Option(hidden=True, default=TIME_RANGE)

    starttime = DateOption(default=-30)
    endtime = DateOption(default=0)

    minmagnitude = FloatOption(default=5.0)
    maxmagnitude = FloatOption(default=10.0)
    magnitudetype = Option()

    mindepth = FloatOption(default=0)
    maxdepth = FloatOption(default=6800)

    LOCATION_GLOBAL = 'locationGlobal'
    LOCATION_BOX = 'locationRange'
    LOCATION_POINT = 'locationDistanceFromPoint'
    LOCATION_STATIONS = 'locationFromStations'
    location_choice = Option(hidden=True, default=LOCATION_GLOBAL)

    minlatitude = FloatOption(default=-90)
    maxlatitude = FloatOption(default=90)
    minlongitude = FloatOption(default=-180)
    maxlongitude = FloatOption(default=180)

    latitude = FloatOption()
    longitude = FloatOption()
    minradius = FloatOption()
    maxradius = FloatOption(default=30)

    def get_time_options(self):
        if self.time_choice == self.TIME_RANGE:
            return self.get_options(['starttime', 'endtime'])
        else:
            return {}

    def get_location_options(self):
        if self.location_choice == self.LOCATION_BOX:
            return self.get_options(['minlatitude', 'maxlatitude', 'minlongitude', 'maxlongitude'])
        elif self.location_choice == self.LOCATION_POINT:
            return self.get_options(['latitude', 'longitude', 'minradius', 'maxradius'])
        else:
            return {}

    def get_obspy_options(self):
        base_keys = ['minmagnitude', 'maxmagnitude', 'magnitudetype', 'mindepth', 'maxdepth']
        options = self.get_options(keys=base_keys)
        # Need special handling for "All Types" value of magnitudetype
        if options.get('magnitudetype', '').lower().startswith('all'):
            del options['magnitudetype']
        options.update(self.get_time_options())
        options.update(self.get_location_options())
        return options



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

from __future__ import (print_function)

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from setuptools.command.install import install
from setuptools.command.develop import develop
from distutils import log
import importlib

# To use a consistent encoding
from codecs import open
from os import path
import sys

module_name = 'pyweed'

# Generate version string from version tuple in server code
pyweed = importlib.import_module(module_name)
version = pyweed.__version__

here = path.abspath(path.dirname(__file__))

# Get the long description from the description.rst file
with open(path.join(here, 'description.rst'), encoding='utf-8') as f:
    long_description = f.read()

def custom_command(subclass):
    orig_run = subclass.run

    def custom_run(self):
        # Check that this version of python is OK
        if sys.version_info < (3,5):
            sys.exit('Sorry, Python versions earlier than 3.5 are not supported')
        orig_run(self)
        self.announce(
'''###########################################
PyWEED has been installed!
###########################################''', level=log.INFO)

    subclass.run = custom_run
    return subclass

@custom_command
class CustomDevelopCommand(develop):
    pass

@custom_command
class CustomInstallCommand(install):
    pass

setup(
    name='pyweed',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,

    description='A GUI application for discovering and downloading seismic data',
    long_description=long_description,

    # The project's main homepage.
    url='https://iris-edu.github.io/pyweed/',

    # Author details
    author='IRIS DMC',
    author_email='software-owner@iris.washington.edu',

    # Choose your license
    license='LGPLv3',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Environment :: MacOS X',
        'Environment :: Win32 (MS Windows)',
        'Environment :: X11 Applications :: Qt',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Physics',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
    ],

    # What does your project relate to?
    keywords='FDSN IRIS miniSEED',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(), #find_packages(exclude=['contrib', 'docs', 'tests']),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    # NOTE: pip can't be used reliably for dependencies, so this is incomplete
    install_requires=['obspy'],

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        module_name: ['app.icns'],
    },

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'pyweed_build_launcher=%s.build_launcher:build' % module_name,
        ],
        'gui_scripts': [
            'pyweed=%s.pyweed_launcher:launch' % module_name,
        ],
    },

    cmdclass={
        'develop': CustomDevelopCommand,
        'install': CustomInstallCommand,
    },
)



#! env python

import sys
from pyweed import __version__, __app_name__, __pkg_path___
import os
from shutil import copy2
import pkg_resources


def build_mac_launcher():
    """
    Build an .app bundle on Mac
    """
    base_path = "%s.app" % __app_name__

    os.makedirs(os.path.join(base_path, "Contents", "MacOS"), exist_ok=True)
    os.makedirs(os.path.join(base_path, "Contents", "Resources"), exist_ok=True)

    bundle_identifier = "edu.iris.%s" % __app_name__
    app_version_string = "%s %s" % (__app_name__, __version__)

    with open(os.path.join(base_path, "Contents", "Info.plist"), "w") as f:
        f.write("""<?xml version="1.0" encoding="UTF-8"?>
        <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
        <plist version="1.0">
        <dict>
            <key>CFBundleDevelopmentRegion</key>
            <string>English</string>
            <key>CFBundleExecutable</key>
            <string>%s</string>
            <key>CFBundleGetInfoString</key>
            <string>%s</string>
            <key>CFBundleIconFile</key>
            <string>app.icns</string>
            <key>CFBundleIdentifier</key>
            <string>%s</string>
            <key>CFBundleInfoDictionaryVersion</key>
            <string>6.0</string>
            <key>CFBundleName</key>
            <string>%s</string>
            <key>CFBundlePackageType</key>
            <string>APPL</string>
            <key>CFBundleShortVersionString</key>
            <string>%s</string>
            <key>CFBundleSignature</key>
            <string>????</string>
            <key>CFBundleVersion</key>
            <string>%s</string>
            <key>NSAppleScriptEnabled</key>
            <string>YES</string>
            <key>NSMainNibFile</key>
            <string>MainMenu</string>
            <key>NSPrincipalClass</key>
            <string>NSApplication</string>
        </dict>
        </plist>
        """ % (__app_name__, app_version_string, bundle_identifier, __app_name__, app_version_string, __version__))

    with open(os.path.join(base_path, "Contents", "PkgInfo"), "w") as f:
        f.write("APPL????")

    icons_stream = pkg_resources.resource_stream('pyweed', 'app.icns')
    with open(os.path.join(base_path, "Contents", "Resources", "app.icns"), "wb") as f:
        f.write(icons_stream.read())

    # This is the executable script packaged up by setup.py
    conda_bin_path = os.path.dirname(sys.executable)
    bin_src = os.path.join(conda_bin_path, 'pyweed')
    # We will copy it into the app bundle
    bin_dest = os.path.join(base_path, "Contents", "MacOS", __app_name__)
    copy2(bin_src, bin_dest)


def build_windows_launcher():
    """
    Build a batch file launcher for Windows
    """
    bat_file = '%s.bat' % __app_name__
    with open(bat_file, "w") as f:
        f.write("""%s -m pyweed.pyweed_launcher""" % sys.executable)


def build():
    if sys.platform.startswith('win'):
        build_windows_launcher()
    elif sys.platform.startswith('darwin'):
        build_mac_launcher()


if __name__ == '__main__':
    build()



# Custom FigureCanvas for displaying Matplotlib output
#
# https://github.com/krischer/instaseis/blob/master/instaseis/gui/qt4mplcanvas.py

from matplotlib import rc as matplotlibrc
import matplotlib as mpl
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt4agg import \
    FigureCanvasQTAgg as FigureCanvas

mpl.rcParams['font.size'] = 10
matplotlibrc('figure.subplot', left=0.0, right=1.0, bottom=0.0, top=1.0)


class MyQt4MplCanvas(FigureCanvas):
    """
    Class to represent the FigureCanvas widget.
    """
    def __init__(self, parent=None):
        # Standard Matplotlib code to generate the plot
        self.fig = Figure()
        # initialize the canvas where the Figure renders into
        super(MyQt4MplCanvas, self).__init__(self.fig)
        self.setParent(parent)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pyweed/gui/uic/PreferencesDialog.ui'
#
# Created by: PyQt4 UI code generator 4.11.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_PreferencesDialog(object):
    def setupUi(self, PreferencesDialog):
        PreferencesDialog.setObjectName(_fromUtf8("PreferencesDialog"))
        PreferencesDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        PreferencesDialog.resize(287, 241)
        PreferencesDialog.setModal(True)
        self.verticalLayout = QtGui.QVBoxLayout(PreferencesDialog)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.dataCentersGroupBox = QtGui.QGroupBox(PreferencesDialog)
        self.dataCentersGroupBox.setObjectName(_fromUtf8("dataCentersGroupBox"))
        self.formLayout_4 = QtGui.QFormLayout(self.dataCentersGroupBox)
        self.formLayout_4.setObjectName(_fromUtf8("formLayout_4"))
        self.label = QtGui.QLabel(self.dataCentersGroupBox)
        self.label.setObjectName(_fromUtf8("label"))
        self.formLayout_4.setWidget(0, QtGui.QFormLayout.LabelRole, self.label)
        self.eventDataCenterComboBox = QtGui.QComboBox(self.dataCentersGroupBox)
        self.eventDataCenterComboBox.setObjectName(_fromUtf8("eventDataCenterComboBox"))
        self.formLayout_4.setWidget(0, QtGui.QFormLayout.FieldRole, self.eventDataCenterComboBox)
        self.label_2 = QtGui.QLabel(self.dataCentersGroupBox)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.formLayout_4.setWidget(1, QtGui.QFormLayout.LabelRole, self.label_2)
        self.stationDataCenterComboBox = QtGui.QComboBox(self.dataCentersGroupBox)
        self.stationDataCenterComboBox.setObjectName(_fromUtf8("stationDataCenterComboBox"))
        self.formLayout_4.setWidget(1, QtGui.QFormLayout.FieldRole, self.stationDataCenterComboBox)
        self.verticalLayout.addWidget(self.dataCentersGroupBox)
        self.groupBox = QtGui.QGroupBox(PreferencesDialog)
        self.groupBox.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.formLayout_3 = QtGui.QFormLayout(self.groupBox)
        self.formLayout_3.setObjectName(_fromUtf8("formLayout_3"))
        self.label_3 = QtGui.QLabel(self.groupBox)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.formLayout_3.setWidget(0, QtGui.QFormLayout.LabelRole, self.label_3)
        self.cacheSizeSpinBox = QtGui.QSpinBox(self.groupBox)
        self.cacheSizeSpinBox.setMaximum(500)
        self.cacheSizeSpinBox.setObjectName(_fromUtf8("cacheSizeSpinBox"))
        self.formLayout_3.setWidget(0, QtGui.QFormLayout.FieldRole, self.cacheSizeSpinBox)
        self.verticalLayout.addWidget(self.groupBox)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        spacerItem = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.okButton = QtGui.QPushButton(PreferencesDialog)
        self.okButton.setDefault(True)
        self.okButton.setObjectName(_fromUtf8("okButton"))
        self.horizontalLayout.addWidget(self.okButton)
        self.cancelButton = QtGui.QPushButton(PreferencesDialog)
        self.cancelButton.setObjectName(_fromUtf8("cancelButton"))
        self.horizontalLayout.addWidget(self.cancelButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout.setStretch(2, 1)

        self.retranslateUi(PreferencesDialog)
        QtCore.QMetaObject.connectSlotsByName(PreferencesDialog)

    def retranslateUi(self, PreferencesDialog):
        PreferencesDialog.setWindowTitle(_translate("PreferencesDialog", "Preferences", None))
        self.dataCentersGroupBox.setTitle(_translate("PreferencesDialog", "Data Centers", None))
        self.label.setText(_translate("PreferencesDialog", "Events", None))
        self.label_2.setText(_translate("PreferencesDialog", "Stations", None))
        self.groupBox.setTitle(_translate("PreferencesDialog", "Advanced", None))
        self.label_3.setText(_translate("PreferencesDialog", "Cache", None))
        self.cacheSizeSpinBox.setSuffix(_translate("PreferencesDialog", " Mbytes", None))
        self.okButton.setText(_translate("PreferencesDialog", "Save", None))
        self.cancelButton.setText(_translate("PreferencesDialog", "Cancel", None))




# -*- coding: utf-8 -*-
"""
Custom QTableWidgetItem that embeds an image

To be inserted into the QTableWidget with setItem().

http://stackoverflow.com/questions/14365875/qt-cannot-put-an-image-in-a-table

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from PyQt4 import QtCore
from PyQt4 import QtGui
from pyweed.gui.TableWidget import CustomTableWidgetItemMixin


class MyTableWidgetImageItem(CustomTableWidgetItemMixin, QtGui.QTableWidgetItem):
    def __init__(self, imagePath=None):
        super(MyTableWidgetImageItem, self).__init__()
        pic = QtGui.QPixmap(imagePath)
        # NOTE:  From QPixmap we could use scaledToWidth() but the resulting images aren't so pretty
        # pic = pic.scaledToWidth(500)
        self.setData(QtCore.Qt.DecorationRole, pic)
        # NOTE:  From QLabel we could use setScaledContents() but this doesn't preserve the aspect ratio
        # self.setScaledContents(True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Main PyWEED application

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from __future__ import (absolute_import, division, print_function)

# Basic packages
import os
import logging

# Pyweed UI components
from pyweed.preferences import Preferences, user_config_path
from pyweed.pyweed_utils import manage_cache, iter_channels, get_sncl, get_preferred_origin, DataRequest, get_distance
from obspy.clients.fdsn import Client
from pyweed.event_options import EventOptions
from pyweed.station_options import StationOptions
from pyweed.events_handler import EventsHandler
from pyweed.stations_handler import StationsHandler, StationsDataRequest
from PyQt4.QtCore import QObject

LOGGER = logging.getLogger(__name__)


class NoConsoleLoggingFilter(logging.Filter):
    """
    Logging filter that excludes the (very noisy) output from the attached Python console
    """
    exclude = ('ipykernel', 'traitlets',)

    def filter(self, record):
        for exclude in self.exclude:
            if record.name.startswith(exclude):
                return False
        return True


class PyWeedCore(QObject):
    """
    This is intended to be the core PyWEED functionality, without any reference to the GUI layer.
    The original intent was to allow this to run independently, eg. from a script or interactive shell.
    """

    preferences = None

    event_client = None
    event_data_center = None
    event_options = None
    events_handler = None
    events = None
    selected_event_ids = None

    station_client = None
    station_data_center = None
    station_options = None
    stations_handler = None
    stations = None
    selected_station_ids = None

    def __init__(self):
        super(PyWeedCore, self).__init__()
        self.configure_logging()
        self.event_options = EventOptions()
        self.station_options = StationOptions()
        self.load_preferences()
        self.manage_cache()

        self.events_handler = EventsHandler(self)
        self.events_handler.done.connect(self.on_events_loaded)

        self.stations_handler = StationsHandler(self)
        self.stations_handler.done.connect(self.on_stations_loaded)

    def set_event_data_center(self, data_center):
        """
        Set the data center used for events
        """
        if data_center != self.event_data_center or not self.event_client:
            # Use the station client if they're the same, otherwise create a client
            if data_center == self.station_data_center and self.station_client:
                client = self.station_client
            else:
                LOGGER.info("Creating ObsPy client for %s", data_center)
                client = Client(data_center)
            # Verify that this client supports events
            if 'event' not in client.services:
                LOGGER.error("The %s data center does not provide an event service" % data_center)
                return
            # Update settings
            self.event_data_center = data_center
            self.event_client = client

    def set_station_data_center(self, data_center):
        """
        Set the data center used for stations
        """
        if data_center != self.station_data_center or not self.station_client:
            # Use the event client if they're the same, otherwise create a client
            if data_center == self.event_data_center and self.event_client:
                client = self.event_client
            else:
                LOGGER.info("Creating ObsPy client for %s", data_center)
                client = Client(data_center)
            # Verify that this client supports station and dataselect
            for service in ('station', 'dataselect',):
                if service not in client.services:
                    LOGGER.error("The %s data center does not provide a %s service" % (data_center, service))
                    return
            # Update settings
            self.station_data_center = data_center
            self.station_client = client

    def load_preferences(self):
        """
        Load configurable preferences from ~/.pyweed/config.ini
        """
        LOGGER.info("Loading preferences")
        self.preferences = Preferences()
        try:
            self.preferences.load()
        except Exception as e:
            LOGGER.error("Unable to load configuration preferences -- using defaults.\n%s", e)
        self.set_event_options(self.preferences.EventOptions)
        self.set_event_data_center(self.preferences.Data.eventDataCenter)
        self.set_station_options(self.preferences.StationOptions)
        self.set_station_data_center(self.preferences.Data.stationDataCenter)

    def save_preferences(self):
        """
        Save preferences to ~/.pyweed/config.ini
        """
        LOGGER.info("Saving preferences")
        try:
            self.preferences.EventOptions.update(self.event_options.get_options(stringify=True))
            self.preferences.Data.eventDataCenter = self.event_data_center
            self.preferences.StationOptions.update(self.station_options.get_options(stringify=True))
            self.preferences.Data.stationDataCenter = self.station_data_center
            self.preferences.save()
        except Exception as e:
            LOGGER.error("Unable to save configuration preferences: %s", e)

    def configure_logging(self):
        """
        Configure the root logger
        """
        logger = logging.getLogger()
        try:
            log_level = getattr(logging, self.preferences.Logging.level)
            logger.setLevel(log_level)
        except Exception:
            logger.setLevel(logging.DEBUG)
        # Log to the terminal if available, otherwise log to a file
        if False:
            # Log to stderr
            handler = logging.StreamHandler()
        else:
            log_file = os.path.join(user_config_path(), 'pyweed.log')
            handler = logging.FileHandler(log_file, mode='w')
        formatter = logging.Formatter(
            '%(asctime)s - %(levelname)s - %(name)s - %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        )
        handler.setFormatter(formatter)
        handler.addFilter(NoConsoleLoggingFilter())
        logger.addHandler(handler)
        LOGGER.info("Logging configured")

    def manage_cache(self, init=True):
        """
        Make sure the waveform download directory exists and isn't full
        """
        download_path = self.preferences.Waveforms.downloadDir
        cache_size = int(self.preferences.Waveforms.cacheSize)
        LOGGER.info('Checking on download directory...')
        if os.path.exists(download_path):
            manage_cache(download_path, cache_size)
        elif init:
            try:
                os.makedirs(download_path, 0o700)
            except Exception as e:
                LOGGER.error("Creation of download directory failed with" + " error: \"%s\'""" % e)
                raise

    ###############
    # Events
    ###############

    def set_event_options(self, options):
        """
        Update the event options
        """
        LOGGER.debug("Set event options: %s", repr(options))
        self.event_options.set_options(options)

    def get_event_obspy_options(self):
        """
        Get the options for making an event request from Obspy
        """
        return self.event_options.get_obspy_options()

    def fetch_events(self, options=None):
        """
        Launch a fetch operation for events
        """
        if options:
            # Simple request
            request = DataRequest(self.event_client, options)
        else:
            # Potentially complex request
            request = DataRequest(
                self.event_client,
                self.event_options.get_obspy_options()
            )
        self.events_handler.load_catalog(request)

    def on_events_loaded(self, events):
        """
        Handler triggered when the EventsHandler finishes loading events
        """
        if not isinstance(events, Exception):
            self.set_events(events)

    def set_events(self, events):
        """
        Set the current event list

        @param events: a Catalog
        """
        LOGGER.info("Set events")
        self.events = events

    def set_selected_event_ids(self, event_ids):
        self.selected_event_ids = event_ids

    def iter_selected_events(self):
        """
        Iterate over the selected events
        """
        if self.events:
            for event in self.events:
                event_id = event.resource_id.id
                if event_id in self.selected_event_ids:
                    yield event

    def iter_selected_event_locations(self):
        """
        Return an iterator of (id, (lat, lon)) for each event.
        """
        for event in self.iter_selected_events():
            origin = get_preferred_origin(event)
            if origin:
                yield (event.resource_id.id, (origin.latitude, origin.longitude),)

    ###############
    # Stations
    ###############

    def set_station_options(self, options):
        LOGGER.debug("Set station options: %s", repr(options))
        self.station_options.set_options(options)

    def fetch_stations(self, options=None):
        """
        Load stations
        """
        if options:
            # Simple request
            request = DataRequest(self.station_client, options)
        else:
            # Potentially complex request
            request = StationsDataRequest(
                self.station_client,
                self.station_options.get_obspy_options(),
                self.station_options.get_event_distances(),
                self.iter_selected_event_locations()
            )
        self.stations_handler.load_inventory(request)

    def on_stations_loaded(self, stations):
        if not isinstance(stations, Exception):
            self.set_stations(stations)

    def set_stations(self, stations):
        """
        Set the current station list

        @param stations: an Inventory
        """
        LOGGER.info("Set stations")
        self.stations = stations

    def set_selected_station_ids(self, station_ids):
        self.selected_station_ids = station_ids

    def iter_selected_stations(self):
        """
        Iterate over the selected stations (channels)
        Yields (network, station, channel) for each selected channel
        """
        if self.stations:
            for (network, station, channel) in iter_channels(self.stations):
                sncl = get_sncl(network, station, channel)
                if sncl in self.selected_station_ids:
                    yield (network, station, channel)

    ###############
    # Waveforms
    ###############

    def iter_selected_events_stations(self):
        """
        Iterate through the selected event/station combinations.

        The main use case this method is meant to handle is where the user
        loaded stations based on selected events.

        For example, if the user selected multiple events and searched for stations
        within 20 degrees of any event, there may be stations that are within 20 degrees
        of one event but farther away from others -- we want to ensure that we only include
        the event/station combinations that are within 20 degrees of each other.
        """
        events = list(self.iter_selected_events())

        # Look for any event-based distance filter
        distance_range = self.station_options.get_event_distances()

        if distance_range:
            # Event locations by id
            event_locations = dict(self.iter_selected_event_locations())
        else:
            event_locations = {}

        # Iterate through the stations
        for (network, station, channel) in self.iter_selected_stations():
            for event in events:
                if distance_range:
                    event_location = event_locations.get(event.resource_id.id)
                    if event_location:
                        distance = get_distance(
                            event_location[0], event_location[1],
                            station.latitude, station.longitude)
                        LOGGER.debug(
                            "Distance from (%s, %s) to (%s, %s): %s",
                            event_location[0], event_location[1],
                            station.latitude, station.longitude,
                            distance
                        )
                        if distance < distance_range['mindistance'] or distance > distance_range['maxdistance']:
                            continue
                # If we reach here, include this event/station pair
                yield (event, network, station, channel)

    def close(self):
        self.manage_cache(init=False)
        self.save_preferences()


if __name__ == "__main__":
    pyweed = PyWeedCore()
    # Do something?



# -*- coding: utf-8 -*-
"""
Dialog for selecting and downloading waveform data

:copyright:
    Mazama Science, IRIS
:license:
    GNU Lesser General Public License, Version 3
    (http://www.gnu.org/copyleft/lesser.html)
"""

from PyQt4 import QtGui, QtCore
from pyweed.gui.uic import WaveformDialog
from pyweed.waveforms_handler import WaveformsHandler
from logging import getLogger
from pyweed.gui.TableItems import TableItems, Column
from pyweed.pyweed_utils import get_event_name, TimeWindow, OUTPUT_FORMATS, PHASES
from pyweed.preferences import safe_int
from PyQt4.QtGui import QTableWidgetItem
from pyweed.gui.Adapters import ComboBoxAdapter
from pyweed.gui.BaseDialog import BaseDialog
from pyweed.gui.SpinnerWidget import SpinnerWidget
from pyweed.gui.TableWidget import CustomTableWidgetItemMixin

LOGGER = getLogger(__name__)


# Download/save status values
STATUS_READY = "ready"  # Waiting for user to initiate
STATUS_WORKING = "working"  # Working
STATUS_DONE = "done"  # Finished
STATUS_ERROR = "error"  # Something went wrong


class KeepIndicatorTableWidgetItem(CustomTableWidgetItemMixin, QTableWidgetItem):
    """
    Custom QTableWidgetItem that indicates whether the row is included in the request

    Note that this uses Unicode characters for the checkboxes, this appears to be the easiest way to
    control the size, and we don't need an actual toggle widget here since that is done at the row level.
    """
    checkedText = '☑'
    checkedIcon = QtGui.QPixmap(':qrc/check-on.png')
    uncheckedText = '☐'
    uncheckedIcon = QtGui.QPixmap(':qrc/check-off.png')
    fontSize = 36
    checkedBackground = QtGui.QBrush(QtGui.QColor(220, 239, 223))
    uncheckedBackground = QtGui.QBrush(QtCore.Qt.NoBrush)

    def __init__(self, value):
        super(KeepIndicatorTableWidgetItem, self).__init__()
        self.setKeep(value)
        # Use a large font size for the checkboxes
        font = QtGui.QFont()
        font.setPointSize(self.fontSize)
        self.setFont(font)

    def keep(self):
        return self.data(QtCore.Qt.UserRole)

    def setKeep(self, value):
        self.setData(QtCore.Qt.UserRole, value)
        # Update as needed to indicate the value to the user
        if value:
            self.setBackground(self.checkedBackground)
            self.setData(QtCore.Qt.DecorationRole, self.checkedIcon)
        else:
            self.setBackground(self.uncheckedBackground)
            self.setData(QtCore.Qt.DecorationRole, self.uncheckedIcon)


class WaveformTableWidgetItem(CustomTableWidgetItemMixin, QtGui.QTableWidgetItem):
    """
    Custom QTableWidgetItem that shows a waveform image (or a status message)
    """
    imagePath = None
    errorForeground = QtGui.QBrush(QtGui.QColor(255, 0, 0))
    defaultForeground = None

    def __init__(self, waveform):
        super(WaveformTableWidgetItem, self).__init__()
        self.defaultForeground = self.foreground()
        self.setWaveform(waveform)

    def setWaveform(self, waveform):
        if waveform and waveform.error:
            self.setForeground(self.errorForeground)
        else:
            self.setForeground(self.defaultForeground)
        if waveform:
            if waveform.loading or waveform.error or not waveform.image_exists:
                self.imagePath = None
                self.setData(QtCore.Qt.DecorationRole, None)
                if waveform.loading:
                    self.setText('Loading waveform data...')
                elif waveform.error:
                    self.setText(waveform.error)
                else:
                    self.setText('')
            else:
                if waveform.image_path != self.imagePath:
                    self.imagePath = waveform.image_path
                    pic = QtGui.QPixmap(waveform.image_path)
                    self.setData(QtCore.Qt.DecorationRole, pic)
                    self.setText('')
        else:
            self.imagePath = None
            self.setText('')


class WaveformTableItems(TableItems):

    #: Fixed row height based on the height of the waveform image
    rowHeight = 110

    columns = [
        Column('Id'),
        Column('Keep'),
        Column('Event Time', width=100),
        Column('Location', width=100),
        Column('Magnitude'),
        Column('SNCL'),
        Column('Distance'),
        Column('Waveform', width=600),
    ]

    def keepWidget(self, keep, **props):
        return self.applyProps(KeepIndicatorTableWidgetItem(keep), **props)

    def waveformWidget(self, waveform, **props):
        return self.applyProps(WaveformTableWidgetItem(waveform), **props)

    def rows(self, data):
        """
        Turn the data into rows (an iterable of lists) of QTableWidgetItems
        """
        for waveform in data:
            # Make the time and location wrap
            yield [
                self.stringWidget(waveform.waveform_id),
                self.keepWidget(waveform.keep,
                                textAlignment=QtCore.Qt.AlignCenter),
                self.stringWidget(waveform.event_time_str),
                self.stringWidget(waveform.event_description),
                self.numericWidget(waveform.event_mag_value, waveform.event_mag,
                                   textAlignment=QtCore.Qt.AlignCenter),
                self.stringWidget(waveform.sncl,
                                  textAlignment=QtCore.Qt.AlignCenter),
                self.numericWidget(waveform.distance, '%.02f°',
                                   textAlignment=QtCore.Qt.AlignCenter),
                self.waveformWidget(waveform),
            ]


# Convenience values for some commonly used table column values
_COLUMN_NAMES = [c.label for c in WaveformTableItems.columns]
WAVEFORM_ID_COLUMN = _COLUMN_NAMES.index('Id')
WAVEFORM_KEEP_COLUMN = _COLUMN_NAMES.index('Keep')
WAVEFORM_IMAGE_COLUMN = _COLUMN_NAMES.index('Waveform')


class TimeWindowAdapter(QtCore.QObject):
    """
    Adapter tying a set of inputs to a TimeWindow
    """

    # Signal indicating that the time window has changed
    changed = QtCore.pyqtSignal()

    def __init__(self, secondsBeforeSpinBox, secondsAfterSpinBox,
                 secondsBeforePhaseComboBox, secondsAfterPhaseComboBox):
        super(TimeWindowAdapter, self).__init__()

        self.timeWindow = TimeWindow()
        # Phase options
        phaseOptions = [(phase.name, phase.label) for phase in PHASES]

        self.secondsBeforeSpinBox = secondsBeforeSpinBox
        self.secondsAfterSpinBox = secondsAfterSpinBox
        self.secondsBeforePhaseAdapter = ComboBoxAdapter(
            secondsBeforePhaseComboBox, phaseOptions)
        self.secondsAfterPhaseAdapter = ComboBoxAdapter(
            secondsAfterPhaseComboBox, phaseOptions)

        # Connect input signals
        self.secondsBeforeSpinBox.valueChanged.connect(self.onTimeWindowChanged)
        self.secondsAfterSpinBox.valueChanged.connect(self.onTimeWindowChanged)
        self.secondsBeforePhaseAdapter.changed.connect(self.onTimeWindowChanged)
        self.secondsAfterPhaseAdapter.changed.connect(self.onTimeWindowChanged)

    def onTimeWindowChanged(self):
        """
        When an input changes, update the time window and emit a notification
        """
        self.timeWindow.update(
            self.secondsBeforeSpinBox.value(),
            self.secondsAfterSpinBox.value(),
            self.secondsBeforePhaseAdapter.getValue(),
            self.secondsAfterPhaseAdapter.getValue()
        )
        self.changed.emit()

    def setValues(self, start_offset, end_offset, start_phase, end_phase):
        """
        This should be called on startup to set the initial values
        """
        self.timeWindow.update(
            start_offset, end_offset, start_phase, end_phase
        )
        self.updateInputs()

    def updateInputs(self):
        """
        Update the inputs to reflect the current state
        """
        self.secondsBeforeSpinBox.setValue(self.timeWindow.start_offset)
        self.secondsAfterSpinBox.setValue(self.timeWindow.end_offset)
        self.secondsBeforePhaseAdapter.setValue(self.timeWindow.start_phase)
        self.secondsAfterPhaseAdapter.setValue(self.timeWindow.end_phase)


class WaveformDialog(BaseDialog, WaveformDialog.Ui_WaveformDialog):

    tableItems = None

    """
    Dialog window for selection and display of waveforms.
    """
    def __init__(self, pyweed, parent=None):
        super(WaveformDialog, self).__init__(parent=parent)

        LOGGER.debug('Initializing waveform dialog...')

        self.setupUi(self)
        self.setWindowTitle('Waveforms')

        # Keep a reference to globally shared components
        self.pyweed = pyweed

        # Time window for waveform selection
        self.timeWindowAdapter = TimeWindowAdapter(
            self.secondsBeforeSpinBox,
            self.secondsAfterSpinBox,
            self.secondsBeforePhaseComboBox,
            self.secondsAfterPhaseComboBox
        )

        self.saveFormatAdapter = ComboBoxAdapter(
            self.saveFormatComboBox,
            [(f.value, f.label) for f in OUTPUT_FORMATS]
        )

        # Initialize any preference-based settings
        self.loadPreferences()

        # Modify default GUI settings
        self.saveDirectoryPushButton.setText(self.waveformDirectory)
        self.saveDirectoryPushButton.setFocusPolicy(QtCore.Qt.NoFocus)

        # Waveforms
        self.waveforms_handler = WaveformsHandler(LOGGER, pyweed.preferences, pyweed.station_client)
        # The callbacks here are expensive, so use QueuedConnection to run them asynchronously
        self.waveforms_handler.progress.connect(self.onWaveformDownloaded, QtCore.Qt.QueuedConnection)
        self.waveforms_handler.done.connect(self.onAllDownloaded, QtCore.Qt.QueuedConnection)

        # Spinner overlays for downloading and saving
        self.downloadSpinner = SpinnerWidget("Downloading...", parent=self.downloadGroupBox)
        self.downloadSpinner.cancelled.connect(self.onDownloadCancel)
        self.saveSpinner = SpinnerWidget("Saving...", cancellable=False, parent=self.saveGroupBox)

        # Connect signals associated with the main table
        # self.selectionTable.horizontalHeader().sortIndicatorChanged.connect(self.selectionTable.resizeRowsToContents)
        self.selectionTable.itemClicked.connect(self.handleTableItemClicked)

        # Connect the Download and Save GUI elements
        self.downloadPushButton.clicked.connect(self.onDownloadPushButton)
        self.savePushButton.clicked.connect(self.onSavePushButton)
        self.saveDirectoryPushButton.clicked.connect(self.getWaveformDirectory)
        self.saveDirectoryBrowseToolButton.clicked.connect(self.browseWaveformDirectory)

        self.saveFormatAdapter.changed.connect(self.resetSave)

        # Connect signals associated with comboBoxes
        # NOTE:  http://www.tutorialspoint.com/pyqt/pyqt_qcombobox_widget.htm
        # NOTE:  currentIndexChanged() responds to both user and programmatic changes.
        #        Use activated() for user initiated changes
        self.eventComboBox.activated.connect(self.onFilterChanged)
        self.networkComboBox.activated.connect(self.onFilterChanged)
        self.stationComboBox.activated.connect(self.onFilterChanged)

        # Connect the timewindow signals
        self.timeWindowAdapter.changed.connect(self.resetDownload)

        # Dictionary of filter values
        self.filters = {}

        # Information about download progress
        self.downloadCount = 0
        self.downloadCompleted = 0

        LOGGER.debug('Finished initializing waveform dialog')

    # NOTE:  http://stackoverflow.com/questions/12366521/pyqt-checkbox-in-qtablewidget
    # NOTE:  http://stackoverflow.com/questions/30462078/using-a-checkbox-in-pyqt
    @QtCore.pyqtSlot(QTableWidgetItem)
    def handleTableItemClicked(self, item):
        """
        Triggered whenever an item in the waveforms table is clicked.
        """
        row = item.row()

        LOGGER.debug("Clicked on table row")

        # Toggle the Keep state
        waveformID = str(self.selectionTable.item(row, WAVEFORM_ID_COLUMN).text())
        waveform = self.waveforms_handler.get_waveform(waveformID)
        waveform.keep = not waveform.keep
        keepItem = self.selectionTable.item(row, WAVEFORM_KEEP_COLUMN)
        keepItem.setKeep(waveform.keep)

    @QtCore.pyqtSlot()
    def loadWaveformChoices(self):
        """
        Fill the selectionTable with all SNCL-Event combinations selected in the MainWindow.
        This function is triggered whenever the "Get Waveforms" button in the MainWindow is clicked.
        """

        LOGGER.debug('Loading waveform choices...')

        self.resetDownload()

        self.waveforms_handler.create_waveforms(self.pyweed)

        # Add events to the eventComboBox -------------------------------

        self.eventComboBox.clear()

        self.eventComboBox.addItem('All events')
        for event in self.pyweed.iter_selected_events():
            self.eventComboBox.addItem(get_event_name(event))

        # Add networks/stations to the networkComboBox and stationsComboBox ---------------------------

        self.networkComboBox.clear()
        self.networkComboBox.addItem('All networks')

        self.stationComboBox.clear()
        self.stationComboBox.addItem('All stations')

        foundNetworks = set()
        foundStations = set()
        for (network, station, _channel) in self.pyweed.iter_selected_stations():
            if network.code not in foundNetworks:
                foundNetworks.add(network.code)
                self.networkComboBox.addItem(network.code)
            netstaCode = '.'.join((network.code, station.code))
            if netstaCode not in foundStations:
                foundStations.add(netstaCode)
                self.stationComboBox.addItem(netstaCode)

        self.loadSelectionTable()

        # Start downloading data
        self.downloadWaveformData()

        LOGGER.debug('Finished loading waveform choices')

    @QtCore.pyqtSlot()
    def loadSelectionTable(self):
        """
        Add event-SNCL combinations to the selection table
        """

        LOGGER.debug('Loading waveform selection table...')

        # Use WaveformTableItems to put the data into the table
        if not self.tableItems:
            self.tableItems = WaveformTableItems(
                self.selectionTable
            )
        self.tableItems.fill(self.iterWaveforms())

        self.filterSelectionTable()

        LOGGER.debug('Finished loading waveform selection table')

    @QtCore.pyqtSlot(int)
    def onFilterChanged(self):
        self.filters = {}
        self.filterSelectionTable()

    def filterSelectionTable(self):
        """
        Filter the selection table based on the currently defined filters
        """
        if self.tableItems:

            if not self.filters:
                self.filters = {
                    'event': self.eventComboBox.currentText(),
                    'network': self.networkComboBox.currentText(),
                    'station': self.stationComboBox.currentText(),
                }

            filterResults = dict(
                (waveform.waveform_id, self.applyFilter(waveform))
                for waveform in self.iterWaveforms()
            )

            def filterFn(row):
                waveformID = str(self.selectionTable.item(row, WAVEFORM_ID_COLUMN).text())
                return filterResults.get(waveformID)

            self.tableItems.filter(filterFn)

    def iterWaveforms(self, saveable_only=False):
        """
        Iterate through the waveforms, optionally yielding only the saveable ones
        """
        for waveform in self.waveforms_handler.waveforms:
            if saveable_only and not (waveform.keep and waveform.mseed_exists):
                continue
            yield waveform

    def applyFilter(self, waveform):
        """
        Apply self.filters to the given waveform
        @return True iff the waveform should be included
        """
        # Get the values from the waveform to match against the filter value
        sncl_parts = waveform.sncl.split('.')
        net_code = sncl_parts[0]
        netsta_code = '.'.join(sncl_parts[:2])
        filter_values = {
            'event': get_event_name(waveform.event_ref()),
            'network': net_code,
            'station': netsta_code
        }
        for (fname, fval) in self.filters.items():
            if not fval.startswith('All') and fval != filter_values[fname]:
                return False
        return True

    @QtCore.pyqtSlot()
    def onDownloadPushButton(self):
        """
        Triggered when downloadPushButton is clicked.
        """
        if self.waveformsDownloadStatus == STATUS_READY:
            # Start the download
            self.downloadWaveformData()

    @QtCore.pyqtSlot()
    def onDownloadCancel(self):
        if self.waveformsDownloadStatus == STATUS_WORKING:
            # Cancel running download
            self.waveforms_handler.cancel_download()

    def updateToolbars(self):
        """
        Update the UI elements to reflect the current status
        """
        self.downloadPushButton.setEnabled(self.waveformsDownloadStatus == STATUS_READY)

    @QtCore.pyqtSlot()
    def onSavePushButton(self):
        """
        Triggered after savePushButton is toggled.
        """
        if self.waveformsSaveStatus != STATUS_WORKING:
            self.waveformsSaveStatus = STATUS_WORKING
            self.saveSpinner.show()
            if self.waveformsDownloadStatus == STATUS_DONE:
                # If any downloads are complete, we can trigger the save now
                self.saveWaveformData()
            else:
                # Otherwise we have to wait until the download is done, indicate this to the user
                self.saveSpinner.setLabel("Waiting for downloads to finish")
                # If not already downloading, try to start
                if self.waveformsDownloadStatus != STATUS_WORKING:
                    self.downloadWaveformData()

    @QtCore.pyqtSlot()
    def resetDownload(self):
        """
        This function is triggered whenever the values in secondsBeforeSpinBox or
        secondsAfterSpinBox are changed. Any change means that we need to wipe out
        all the downloads that have occurred and start over.
        """
        LOGGER.debug("Download button reset")
        self.waveformsDownloadStatus = STATUS_READY
        self.resetSave()

    @QtCore.pyqtSlot()
    def resetSave(self):
        """
        This function is triggered whenever the values in saveDirectory or
        saveFormat elements are changed. Any change means that we need to
        start saving from the beginning.
        """
        LOGGER.debug("Save button reset")
        self.waveformsSaveStatus = STATUS_READY
        self.updateToolbars()

    def downloadWaveformData(self):
        """
        This function is triggered after the selectionTable is initially loaded
        by loadWaveformChoices() and, after that, by handleWaveformResponse() after
        it has finished handling a waveform.

        This function looks at the current selectionTable view for any waveforms
        that have not yet been downloaded. After that table is exhausted, it goes
        through all not-yet-downloaded data in waveformHandler.currentDF.
        """

        LOGGER.info("Starting download of waveform data")

        self.waveformsDownloadStatus = STATUS_WORKING
        self.downloadCount = len(self.waveforms_handler.waveforms)
        self.downloadCompleted = 0
        self.downloadSpinner.show()
        self.updateToolbars()

        # Priority is given to waveforms shown on the screen
        priority_ids = [waveform.waveform_id for waveform in self.waveforms_handler.waveforms]
        other_ids = []

        self.waveforms_handler.download_waveforms(
            priority_ids, other_ids, self.timeWindowAdapter.timeWindow)

        # Update the table rows
        for row in range(self.selectionTable.rowCount()):
            waveform_id = self.selectionTable.item(row, WAVEFORM_ID_COLUMN).text()
            waveform = self.waveforms_handler.waveforms_by_id.get(waveform_id)
            self.selectionTable.item(row, WAVEFORM_IMAGE_COLUMN).setWaveform(waveform)

    def getTableRow(self, waveform_id):
        """
        Get the table row for a given waveform
        """
        for row in range(self.selectionTable.rowCount()):
            if self.selectionTable.item(row, WAVEFORM_ID_COLUMN).text() == waveform_id:
                return row
        return None

    @QtCore.pyqtSlot(object)
    def onWaveformDownloaded(self, result):
        """
        Called each time a waveform request has completed
        """
        waveform_id = result.waveform_id
        LOGGER.debug("Ready to display waveform %s (%s)", waveform_id, QtCore.QThread.currentThreadId())

        self.downloadCompleted += 1
        msg = "Downloaded %d of %d" % (self.downloadCompleted, self.downloadCount)
        # self.downloadStatusLabel.setText(msg)
        self.downloadSpinner.setLabel(msg)

        row = self.getTableRow(waveform_id)
        if row is None:
            LOGGER.error("Couldn't find a row for waveform %s", waveform_id)
            return

        waveform = self.waveforms_handler.waveforms_by_id.get(waveform_id)
        self.selectionTable.item(row, WAVEFORM_IMAGE_COLUMN).setWaveform(waveform)

        LOGGER.debug("Displayed waveform %s", waveform_id)

    @QtCore.pyqtSlot(object)
    def onAllDownloaded(self, result):
        """
        Called after all waveforms have been downloaded
        """
        LOGGER.debug('COMPLETED all downloads')

        if self.waveformsDownloadStatus == STATUS_WORKING:
            self.downloadSpinner.hide()

            # If normal result, mark as done. If an error, mark as ready (ie. user can download again)
            if isinstance(result, Exception):
                self.waveformsDownloadStatus = STATUS_READY
            else:
                self.waveformsDownloadStatus = STATUS_DONE
                self.downloadStatusLabel.setText("Downloaded %d waveforms" % len(self.waveforms_handler.waveforms))

                # Initiate save if that was queued
                if self.waveformsSaveStatus == STATUS_WORKING:
                    self.saveWaveformData()

            self.updateToolbars()

    @QtCore.pyqtSlot()
    def saveWaveformData(self):
        """
        Save waveforms after all downloads are complete.
        """

        # Update status
        self.waveformsSaveStatus = STATUS_WORKING
        self.updateToolbars()

        # Update GUI in case we came from an internal call
        QtGui.QApplication.processEvents()

        errors = []
        savedCount = 0
        skippedCount = 0

        try:
            waveforms = self.iterWaveforms(saveable_only=True)
            outputDir = self.waveformDirectory
            outputFormat = self.saveFormatAdapter.getValue()

            for result in self.waveforms_handler.save_waveforms_iter(outputDir, outputFormat, waveforms):
                if isinstance(result.result, Exception):
                    LOGGER.error("Failed to save waveform %s: %s", result.waveform_id, result.result)
                    errors.append("%s: %s" % (result.waveform_id, result.result))
                elif result.result:
                    savedCount += 1
                    self.saveSpinner.setLabel("Saved %d waveforms" % savedCount)
                    QtGui.QApplication.processEvents()  # update GUI
                else:
                    skippedCount += 1
            self.saveStatusLabel.setText("Saved %d waveforms" % savedCount)
            self.saveStatusLabel.repaint()

            LOGGER.info("Save complete: %d saved, %d already existed, %d errors", savedCount, skippedCount, len(errors))

            if errors:
                # Truncate the list of errors if it's very long
                errorCount = len(errors)
                if errorCount > 20:
                    errors = errors[:20]
                    errors.append("(see log for full list)")
                raise Exception("%d waveforms couldn't be saved:\n%s" % (errorCount, "\n".join(errors)))

            self.waveformsSaveStatus = STATUS_DONE
        except Exception as e:
            LOGGER.error(e)
            self.waveformsSaveStatus = STATUS_ERROR
            QtGui.QMessageBox.critical(
                self,
                "Error",
                str(e)
            )
        finally:
            self.updateToolbars()

        self.saveSpinner.hide()
        LOGGER.debug('COMPLETED saving all waveforms')

    @QtCore.pyqtSlot()
    def getWaveformDirectory(self):
        """
        This function is triggered whenever the user presses the "to <directory>" button.
        """
        # If the user quits or cancels this dialog, '' is returned
        newDirectory = str(QtGui.QFileDialog.getExistingDirectory(
            self,
            "Waveform Directory",
            self.waveformDirectory,
            QtGui.QFileDialog.ShowDirsOnly))

        if newDirectory != '':
            self.waveformDirectory = newDirectory
            self.saveDirectoryPushButton.setText(self.waveformDirectory)
            self.resetSave()

    @QtCore.pyqtSlot()
    def browseWaveformDirectory(self):
        """
        This function is triggered whenever the user presses the "browse" button.
        """
        url = QtCore.QUrl.fromLocalFile(self.waveformDirectory)
        QtGui.QDesktopServices.openUrl(url)

    def loadPreferences(self):
        """
        Load preferences relevant to this widget
        """
        prefs = self.pyweed.preferences

        self.waveformDirectory = prefs.Waveforms.saveDir

        self.timeWindowAdapter.setValues(
            safe_int(prefs.Waveforms.timeWindowBefore, 60),
            safe_int(prefs.Waveforms.timeWindowAfter, 600),
            prefs.Waveforms.timeWindowBeforePhase,
            prefs.Waveforms.timeWindowAfterPhase
        )

        self.saveFormatAdapter.setValue(prefs.Waveforms.saveFormat)

    def savePreferences(self):
        """
        Save preferences related to the controls on this widget
        """
        prefs = self.pyweed.preferences

        prefs.Waveforms.saveDir = self.waveformDirectory
        timeWindow = self.timeWindowAdapter.timeWindow
        prefs.Waveforms.timeWindowBefore = timeWindow.start_offset
        prefs.Waveforms.timeWindowAfter = timeWindow.end_offset
        prefs.Waveforms.timeWindowBeforePhase = timeWindow.start_phase
        prefs.Waveforms.timeWindowAfterPhase = timeWindow.end_phase

        prefs.Waveforms.saveFormat = self.saveFormatAdapter.getValue()
